import sys, os
from collections import Counter
import pandas as pd
import csv
from os.path import exists
import glob
import numpy as np
# import pandas as pd
# import locale
# locale.setlocale(locale.LC_ALL, '')
path_files=os.listdir(sys.argv[1])
ext1=sys.argv[2]
ext2=sys.argv[3]
ext_file=sys.argv[4]
run=sys.argv[5]
out=sys.argv[9]


stats_df=pd.read_csv(sys.argv[6], sep=',').drop_duplicates()
lin_df=pd.read_csv(sys.argv[7], sep=',').drop_duplicates()
count_df_r=pd.read_csv(sys.argv[8], sep=',').drop_duplicates()

stats_df=stats_df[(stats_df.tax_id.isin(lin_df.tax_id))]
count_df=count_df_r[(count_df_r.qseqid.isin(stats_df.qseqid))]
stats_df=stats_df[(stats_df.qseqid.isin(count_df.qseqid))]



df_s_l=stats_df.merge(lin_df, on='tax_id', how='left')

df=df_s_l.merge(count_df, on='qseqid', how='left')
df = df[df.iloc[: , -1].notna()]

tmp_df=df.iloc[:, np.r_[0,30:len(df. columns)]]
tmp_df.to_csv(r'tmp_df.csv')


df.to_csv(r'df.csv')
csv_columns=['All-sample']
list_files=[]
for input_files in path_files :
	if input_files.endswith(ext1+ext_file):
		file_name=input_files.split(ext1+ext_file)[0]
		csv_columns.append(file_name)
		list_files.append(file_name)

result_table = pd.DataFrame(columns=csv_columns,index=["Total read R1","Total read R2","Avg read length R1","Avg read length R2","Reads 1 cleaned","Reads 2 cleaned","Avg cleaned read length R1 (after cutadapt)","Avg cleaned read length R2 (after cutadapt)","Avg insert_size","Map on Diptera","UnMapped on Diptera","Map on bacteria","UnMapped on bacteria","Total pairs","Combined pairs","Widows","Assembled","Duplicates","Reads_count","Singletons","Nb of contigs","Min contigs length","Max contigs length","Avg contigs length","Contigs with viral hit","Nb of reads with viral hit 'inside' a contig","Nb of reads with viral hit as singleton","Reads with viral hit","% viral reads","Nb of viral hit 1 =< Reads < 10 ","Nb of viral hit 10 =< Reads < 100 ","Nb of viral hit 100 =< Reads < 1000","Nb of viral hit 1000 =< Reads < 10000","Nb of viral hit 10000=< Reads","Nb viral family","Nb viral genus","Nb viral species"])

contig_length=pd.read_csv("logs/logsAssembly/"+run+"_assembly_stats.txt", sep='	')
stats_contigs=contig_length.iloc[:, 1].describe()

contig_length.columns.values[0] = "qseqid"

n=0
for files_name in list_files :
	with open("logs/logscutadapt/"+files_name+"_cut1.log",'r') as cut1_1:
		for line in cut1_1:
			if "Total reads processed" in line:
				c = line.replace(',','').replace(' ','').replace('\n','').split(":")
				result_table.at['Total read R1', files_name] =int(c[-1])
			if "Total written" in line:
				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
				result_table.at['Avg read length R1', files_name]="%.2f" % (int(c1[1])/int(c[-1]))
	cut1_1.close()
	# with open("logs/logscutadapt/"+files_name+ext2+"_cut1.log",'r') as cut1_2:
	# 	for line in cut1_2:
	# 		if "Total reads processed" in line:
	# 			c = line.replace(',','').replace(' ','').replace('\n','').split(":")
	# 			result_table.at['Total read R2', files_name]=int(c[-1])
	# 		if "Total written" in line:
	# 			c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
	# 			result_table.at['Avg read length R2', files_name]="%.2f" % (int(c1[1])/int(c[-1]))
	# cut1_2.close()
	with open("logs/logscutadapt/"+files_name+"_1_cut2.log",'r') as cut2_1:
		for line in cut2_1:
			if "Reads written" in line:
				c = line.replace(',','').replace('(',':').replace('\n','').replace(' ','').split(":")
				result_table.at['Reads 1 cleaned', files_name]=int(c[2])
			if "Total written" in line:
				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
				result_table.at['Avg cleaned read length R1 (after cutadapt)', files_name]="%.2f" % (int(c1[1])/int(c[2]))

	cut2_1.close()
	# with open("logs/logscutadapt/"+files_name+ext2+"_cut2.log",'r') as cut2_2:
	# 	for line in cut2_2:
	# 		if "Reads written" in line:
	# 			c = line.replace(',','').replace('(',':').replace('\n','').replace(' ','').split(":")
	# 			result_table.at['Reads 2 cleaned', files_name]=int(c[2])
	# 		if "Total written" in line:
	# 			c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
	# 			result_table.at['Avg cleaned read length R2 (after cutadapt)', files_name]="%.2f" % (int(c1[1])/int(c[2]))
	# cut2_2.close()
	# path="logs/insert_size/"+files_name+"_insert_size_metrics_"+run+".txt"
	# files_insert=glob.glob(path)
	# with open(files_insert[0],'r') as insert:
	# 	for line in insert:
	# 		if "LIBRARY	READ_GROUP" in line:
	# 			line = next(insert)
	# 			c = line.split("\t")
	# 			result_table.at['Avg insert_size', files_name]="%.2f" % (float(c[5]))
	# insert.close()
	# with open("logs/logsFLASH/"+files_name+"_flash.log",'r') as flash:
	# 	for line in flash:
	# 		if "Total pairs" in line:
	# 			c = line.replace('\n','').replace(' ','').split(":")
	# 			result_table.at['Total pairs', files_name]=int(c[1])
	# 		if "Combined pairs" in line:
	# 			c = line.replace('\n','').replace(' ','').split(":")
	# 			result_table.at['Combined pairs', files_name]=int(c[1])
	# flash.close()

	path="logs/logs_coverage/"+files_name+"_coverage*"
	files_cov=glob.glob(path)
	with open(files_cov[0],'r') as cov:
		for line in cov:
			if "Mapped" in line:
				c = line.split(":")
				result_table.at['Reads_count', files_name]=int(c[1])
				line = next(cov)
				c = line.split(" ")
				result_table.at['Nb of contigs', files_name]=int(c[0])
				line = next(cov)
				c = line.split(":")
				result_table.at['Singletons', files_name]=int(c[1])
	cov.close()

	with open("logs/logsDuplicates/"+files_name+"_duplicates_pairs_"+run+".txt",'r') as dp:
		for line in dp :
			if "READ:" in line:
				reads_pair = int(line.replace('\n','').split(": ")[-1])
			if "DUPLICATE TOTAL:" in line:
				dup_pair = int(line.replace('\n','').split(": ")[-1])
		result_table.at['Duplicates', files_name]=dup_pair
		result_table.at['Assembled', files_name]=result_table.at['Duplicates', files_name]+result_table.at['Reads_count', files_name]

	with open("logs/logs_contaminent/Stats_contaminent_"+files_name+".txt",'r') as conta:
		for line in conta:
			if "host_pairs_R1" in line:
				c = line.replace('\n','').split(":")
				R1_h=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				R2_h=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				wi_h=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				R1_b=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				R2_b=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				wi_b=c[1]

				result_table.at['Widows', files_name]=int(wi_b)
				result_table.at['UnMapped on Diptera', files_name]=int(R1_h)+int(R2_h)+int(wi_h)
				result_table.at['UnMapped on bacteria', files_name]=int(R1_b)+int(R2_b)+int(wi_b)
				result_table.at['Map on Diptera', files_name]=int(result_table.at['Reads 1 cleaned', files_name])-int(result_table.at['UnMapped on Diptera', files_name])

				result_table.at['Map on bacteria', files_name]=result_table.at['UnMapped on Diptera', files_name]-result_table.at['UnMapped on bacteria', files_name]
	conta.close()

	result_table.at['Nb viral family', files_name]=df[["family", files_name]][df[files_name]> 1].family.value_counts().count()
	result_table.at['Nb viral genus', files_name]=df[["genus", files_name]][df[files_name]> 1].genus.value_counts().count()
	result_table.at['Nb viral species', files_name]=df[["species", files_name]][df[files_name]> 1].species.value_counts().count()
	raws = df[["species",files_name]].groupby('species',as_index=False).agg({files_name: 'sum' }).drop(columns='species')
	print(raws)
	ranges = [1,10,100,1000,10000]
	result_table.at['Reads with viral hit', files_name]=raws.values.sum()
	result_table.at['Nb of viral hit 1 =< Reads < 10 ', files_name]=raws.groupby(pd.cut(raws[files_name], ranges)).count().iloc[0][files_name]
	result_table.at['Nb of viral hit 10 =< Reads < 100 ', files_name]=raws.groupby(pd.cut(raws[files_name], ranges)).count().iloc[1][files_name]
	result_table.at['Nb of viral hit 100 =< Reads < 1000', files_name]=raws.groupby(pd.cut(raws[files_name], ranges)).count().iloc[2][files_name]
	result_table.at['Nb of viral hit 1000 =< Reads < 10000', files_name]=raws.groupby(pd.cut(raws[files_name], ranges)).count().iloc[3][files_name]
	result_table.at['Nb of viral hit 10000=< Reads', files_name]=raws[raws[files_name]> 10000].count().values[0]
	result_table.at['Contigs with viral hit', files_name]=count_df[count_df[files_name]> 0].count().values[0]
	print(result_table.at['Contigs with viral hit', files_name])
	result_table.at['Min contigs length', files_name]=int(contig_length[(contig_length.qseqid.isin(count_df_r[count_df_r[files_name]!= 0][["qseqid",files_name]] .qseqid))].iloc[:, 1].describe().iloc[3])

	result_table.at['Max contigs length', files_name]=int(contig_length[(contig_length.qseqid.isin(count_df_r[count_df_r[files_name]!= 0][["qseqid",files_name]] .qseqid))].iloc[:, 1].describe().iloc[7])

	result_table.at['Avg contigs length', files_name]=round(contig_length[(contig_length.qseqid.isin(count_df_r[count_df_r[files_name]!= 0][["qseqid",files_name]] .qseqid))].iloc[:, 1].describe().iloc[1],2)
	result_table.at["Nb of reads with viral hit 'inside' a contig", files_name]=count_df[count_df[["qseqid",files_name]].qseqid.astype(str).str.startswith('k') | count_df[["qseqid",files_name]].qseqid.astype(str).str.startswith('C')][files_name].sum()
	result_table.at["Nb of reads with viral hit as singleton", files_name]=count_df[~count_df[["qseqid",files_name]].qseqid.astype(str).str.startswith('k') & ~count_df[["qseqid",files_name]].qseqid.astype(str).str.startswith('C')][files_name].sum()
	result_table.at['% viral reads', files_name]=round(result_table.loc["Reads with viral hit", files_name] *100 / (result_table.loc["Total read R1", files_name]+result_table.loc["Total read R1", files_name]),2)
result_table.at['Nb of contigs', 'All-sample']=int(count_df_r.shape[0])-1
result_table.at['Min contigs length', 'All-sample']=int(stats_contigs.iloc[3])
result_table.at['Max contigs length', 'All-sample']=int(stats_contigs.iloc[7])
result_table.at['Avg contigs length', 'All-sample']=round(stats_contigs.iloc[1],2)
result_table.at['Contigs with viral hit', 'All-sample']=count_df.shape[0]
result_table.at['Reads with viral hit', 'All-sample']=count_df.iloc[:, 1:].values.sum()
result_table.at['Nb viral family', 'All-sample']=df.family.value_counts().count()
result_table.at['Nb viral genus', 'All-sample']=df.genus.value_counts().count()
result_table.at['Nb viral species', 'All-sample']=df.species.value_counts().count()

ranges = [1,10,100,1000,10000]
spp=df.iloc[:,np.r_[df.columns.get_loc("species"), df_s_l.shape[1]:df.shape[1]]].groupby('species',as_index=False).agg('sum').drop(columns='species').sum(axis=1)
result_table.at['Nb of viral hit 1 =< Reads < 10 ',  'All-sample']=spp.groupby(pd.cut(spp, ranges)).count().iloc[0]
result_table.at['Nb of viral hit 10 =< Reads < 100 ',  'All-sample']=spp.groupby(pd.cut(spp, ranges)).count().iloc[1]
result_table.at['Nb of viral hit 100 =< Reads < 1000',  'All-sample']=spp.groupby(pd.cut(spp, ranges)).count().iloc[2]
result_table.at['Nb of viral hit 1000 =< Reads < 10000',  'All-sample']=spp.groupby(pd.cut(spp, ranges)).count().iloc[3]
result_table.at['Nb of viral hit 10000=< Reads',  'All-sample']=spp[spp > 10000].count()


row_sum=["Total read R1","Reads 1 cleaned","Map on Diptera","UnMapped on Diptera","Map on bacteria","UnMapped on bacteria","Assembled","Duplicates","Reads_count","Singletons","Nb of reads with viral hit 'inside' a contig","Nb of reads with viral hit as singleton","Reads with viral hit"]

row_mean=["Avg read length R1","Avg cleaned read length R1 (after cutadapt)"]
print("ok3")

for row in row_sum:
	result_table.at[row,'All-sample']=result_table.loc[row,result_table.columns != 'All-sample'].apply(int).sum(axis=0)
for row in row_mean:
	result_table.at[row,'All-sample']=round(result_table.loc[row,result_table.columns != 'All-sample'].apply(float).mean(axis=0),2)



result_table.at['% viral reads',  'All-sample']=round(result_table.loc["Reads with viral hit", 'All-sample']*100 / (result_table.loc["Total read R1", 'All-sample']+result_table.loc["Total read R1", 'All-sample']),2)
result_table.fillna(0, inplace=True)

result_table.to_csv(out)

	#	raws = df[[files_name,'qseqid']]
#	in_contigs=0
#	numcont=0
#	for index, raw in raws.iterrows():
#		numcont+=1
#		if raw[files_name] != 0 :
#			if raw['qseqid'].startswith('k') or raw['qseqid'].startswith('C'):
#				in_contigs+=raw[files_name]
#	sum0=sum10=sum100=sum1000=sum10000=sum0c=0
#	numcont=0
#	for index, raw in raws.iterrows():
#		numcont+=1
#		if raw[files_name] != 0 :
#			sum0+=raw[files_name]
#		if 10 <= raw[files_name] < 100 :
#			sum10+=1
#		if 100 <= raw[files_name] <1000 :
#			sum100+=1
#		if 1000 <= raw[files_name] < 10000 :
#			sum1000+=1
#		if raw[files_name] >= 10000 :
#			sum10000+=1
#	print("ok2.02")
#	result_table.at['Reads with viral hit', files_name]=sum0
#	result_table.at['Nb of viral hit 10 =< Reads < 100 ', files_name]=sum10
#	result_table.at['Nb of viral hit 100 =< Reads < 1000', files_name]=sum100
#	result_table.at['Nb of viral hit 1000 =< Reads < 10000', files_name]=sum1000
#	result_table.at['Nb of viral hit 10000=< Reads', files_name]=sum10000
#	raws = df[[files_name,'qseqid']]
#	in_contigs=0
#	numcont=0
#	for index, raw in raws.iterrows():
#		numcont+=1
#		if raw[files_name] != 0 :
#			if raw['qseqid'].startswith('k') or raw['qseqid'].startswith('C'):
#				in_contigs+=raw[files_name]
#	out_oncitgs=sum0-in_contigs
#	result_table.at['Contigs with viral hit', 'All-sample']=numcont
#	result_table.at["Nb of reads with viral hit 'inside' a contig", files_name]=in_contigs
#	result_table.at["Nb of reads with viral hit as singleton", files_name]=out_oncitgs
##	for i in range(19,24):
##		dict_data[i][files_name]='-'
##	for i in range(24,27):
##		dict_data[i]['All-sample']+=dict_data[i][files_name]

#print("ok2")

#tot_hit=tot_hit10=tot_hit100=tot_hit1000=tot_hit10000=0


#for index, raw in tmp_df.iterrows():
#	sum_raw=0
#	for input_files in path_files :
#		if input_files.endswith(ext1+ext_file):
#			files_name=input_files.split(ext1)[0]
#			sum_raw+=raw[files_name]
#			tot_hit+=1
#	if 10 <= sum_raw< 100 :
#		tot_hit10+=1
#	if 100 <= sum_raw < 1000 :
#		tot_hit100+=1
#	if 1000 <= sum_raw < 10000 :
#		tot_hit1000+=1
#	if sum_raw >= 10000 :
#		tot_hit10000+=1

#result_table.at['Nb of viral hit 10 =< Reads < 100 ','All-sample']=tot_hit10
#result_table.at['Nb of viral hit 100 =< Reads < 1000','All-sample']=tot_hit100
#result_table.at['Nb of viral hit 1000 =< Reads < 10000','All-sample']=tot_hit1000
#result_table.at['Nb of viral hit 10000=< Reads','All-sample']=tot_hit10000

##print("3")
#path=os.listdir("logs/logsAssembly/")
#for input_files in path :
#	if input_files.endswith(run+"_assembly_stats.txt"):
#		with open("logs/logsAssembly/"+input_files,'r') as asb:
#			data=pd.read_csv(asb, sep="\t", header=None)
#			result_table.at['Nb total of contigs','All-sample']=len(data.index)
#			c=data.min()
#			result_table.at['Min contigs length','All-sample']=c.values[1]
#			c=data.max()
#			result_table.at['Max contigs length','All-sample']=c.values[1]
#			c=data.mean()
#			result_table.at['Avg contigs length','All-sample']="%.2f" %(c.values[0])

#row_sum=["Total read R1","Total read R2","Reads 1 cleaned","Reads 2 cleaned","Map on Diptera","UnMapped on Diptera","Map on bacteria","UnMapped on bacteria","Total pairs","Combined pairs","Widows","Assembled","Singletons","Nb of reads with viral hit 'inside' a contig","Nb of reads with viral hit as singleton","Reads with viral hit"]

#row_mean=["Avg read length R1","Avg read length R2","Avg cleaned read length R1 (after cutadapt)","Avg cleaned read length R2 (after cutadapt)","Avg insert_size","Reads spread across n contig (nb for each samples and Avg for all_samples)"]
#print("ok3")

#for row in row_sum:
#	result_table.at[row,'All-sample']=result_table.loc[row,result_table.columns != 'All-sample'].apply(int).sum(axis=0)
#for row in row_mean:
#	result_table.at[row,'All-sample']=result_table.loc[row,result_table.columns != 'All-sample'].apply(float).mean(axis=0).round(2)













#result_table.at['Total read R1','All-sample']=result_table.loc['Total read R1',result_table.columns != 'All-sample'].apply(int).sum(axis=0)
#result_table.at['Total read R2','All-sample']=result_table.loc['Total read R2',result_table.columns != 'All-sample'].apply(int).sum(axis=0)
#result_table.at['Avg read length R1','All-sample']=result_table.loc['Avg read length R1',result_table.columns != 'All-sample'].apply(float).mean(axis=0).round(2)
#result_table.at['Avg read length R2','All-sample']=result_table.loc['Avg read length R2',result_table.columns != 'All-sample'].apply(float).mean(axis=0).round(2)
#result_table.at['Reads 1 cleaned','All-sample']=result_table.loc['Reads 1 cleaned',result_table.columns != 'All-sample'].apply(int).sum(axis=0)
#result_table.at['Reads 2 cleaned','All-sample']=result_table.loc['Reads 2 cleaned',result_table.columns != 'All-sample'].apply(int).sum(axis=0)
#result_table.at['Avg cleaned read length R1 (after cutadapt)','All-sample']=result_table.loc['Avg cleaned read length R1 (after cutadapt)',result_table.columns != 'All-sample'].apply(float).mean(axis=0).round(2)
#result_table.at['Avg cleaned read length R2 (after cutadapt)','All-sample']=result_table.loc['Avg cleaned read length R2 (after cutadapt)',result_table.columns != 'All-sample'].apply(float).mean(axis=0).round(2)
#result_table.at['Avg read length R2','All-sample']=result_table.loc['Avg read length R2',result_table.columns != 'All-sample'].apply(float).mean(axis=0).round(2)
#result_table.at['','All-sample']=result_table.loc[''].sum(axis=0)
#result_table.at['','All-sample']=result_table.loc[''].sum(axis=0)
#result_table.at['','All-sample']=result_table.loc[''].sum(axis=0)
#result_table.at['','All-sample']=result_table.loc[''].sum(axis=0)
#result_table.at['','All-sample']=result_table.loc[''].sum(axis=0)
#result_table.at['','All-sample']=result_table.loc[''].sum(axis=0)
#result_table.at['','All-sample']=result_table.loc[''].sum(axis=0)

#print(result_table)
#result_table.loc['Avg read length R1'].apply(np.round)
#result_table.at['', files_name]

#def WriteDictToCSV(csv_file,csv_columns,dict_data):
#    with open(csv_file, 'w') as csvfile:
#        writer = csv.DictWriter(csvfile, fieldnames=csv_columns)
#        writer.writeheader()
#        for data in dict_data:
#            writer.writerow(data)
#
#
#csv_columns=['Stats','All-sample']
#dict_data = [{'Stats':"Total read R1"},{'Stats':"Total read R2"},{'Stats':"Avg read length R1"},{'Stats':"Avg read length R2"},{'Stats':"Reads 1 cleaned"},{'Stats':"Reads 2 cleaned"},
#{'Stats':"Avg cleaned read length R1 (after cutadapt)"},{'Stats':"Avg cleaned read length R2 (after cutadapt)"},{'Stats':"Avg insert_size"},{'Stats':"Map on Diptera"},{'Stats':"UnMapped on Diptera"},{'Stats':"Map on bacteria"},{'Stats':"UnMapped on bacteria"},
#{'Stats':"Total pairs"},{'Stats':"Combined pairs"},{'Stats':"Widows"},{'Stats':"Assembled"},{'Stats':"Reads spread across n contig (nb for each samples and Avg for all_samples)"},{'Stats':"Singletons"},{'Stats':"Nb total of contigs"},
#{'Stats':"Min contigs length"},{'Stats':"Max contigs length"},{'Stats':"Avg contigs length"},{'Stats':"Contigs with viral hit"},{'Stats':"Nb of reads with viral hit 'inside' a contig"},{'Stats':"Nb of eads with viral hit as singleton"},{'Stats':"Reads with viral hit"},
#{'Stats':"Nb of viral hit 10 =< Reads < 100 "},{'Stats':"Nb of viral hit 100 =< Reads < 1000"},{'Stats':"Nb of viral hit 1000 =< Reads < 10000"},{'Stats':"Nb of viral hit 10000=< Reads"},{'Stats':"Nb viral family"},{'Stats':"Nb viral genus"},{'Stats':"Nb viral species"}]

#for i in range(27):
#	dict_data[i]['All-sample']=int(0)
#list_files=[]
#for input_files in path_files :
#	if input_files.endswith(ext1+ext_file):
#		file_name=input_files.split(ext1+ext_file)[0]
#		csv_columns.append(file_name)
#		list_files.append(file_name)

#for files_name in list_files :
#	with open("logs/logscutadapt/"+files_name+ext1+"_cut1.log",'r') as cut1_1:
#		for line in cut1_1:
#			if "Total reads processed" in line:
#				c = line.replace(',','').replace(' ','').replace('\n','').split(":")
#				dict_data[0][files_name]=int(c[-1])
#			if "Total written" in line:
#				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
#				dict_data[2][files_name]="%.2f" % (int(c1[1])/int(c[-1]))

#	with open("logs/logscutadapt/"+files_name+ext2+"_cut1.log",'r') as cut1_2:
#		for line in cut1_2:
#			if "Total reads processed" in line:
#				c = line.replace(',','').replace(' ','').replace('\n','').split(":")
#				dict_data[1][files_name]=int(c[-1])
#			if "Total written" in line:
#				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
#				dict_data[3][files_name]="%.2f" % (int(c1[1])/int(c[-1]))
#	with open("logs/logscutadapt/"+files_name+ext1+"_cut2.log",'r') as cut2_1:
#		for line in cut2_1:
#			if "Reads written" in line:
#				c = line.replace(',','').replace('(',':').replace('\n','').replace(' ','').split(":")
#				dict_data[4][files_name]=int(c[2])
#			if "Total written" in line:
#
#				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
#				dict_data[6][files_name]="%.2f" % (int(c1[1])/int(c[2]))
#	with open("logs/logscutadapt/"+files_name+ext2+"_cut2.log",'r') as cut2_2:
#		for line in cut2_2:
#			if "Reads written" in line:
#				c = line.replace(',','').replace('(',':').replace('\n','').replace(' ','').split(":")
#				dict_data[5][files_name]=int(c[2])
#			if "Total written" in line:
#				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
#				dict_data[7][files_name]="%.2f" % (int(c1[1])/int(c[2]))

#	path="logs/insert_size/"+files_name+"_insert_size_metrics_"+run+".txt"
#	files_insert=glob.glob(path)
#	with open(files_insert[0],'r') as insert:
##	with open("logs/insert_size/"+files_name+"_insert_size_metrics.txt",'r') as insert:
#		for line in insert:
#			if "LIBRARY	READ_GROUP" in line:
#				line = next(insert)
#				c = line.split("\t")
#				dict_data[8][files_name]="%.2f" % (float(c[4]))

#	with open("logs/logsFLASH/"+files_name+"_flash.log",'r') as flash:
#		for line in flash:
#			if "Total pairs" in line:
#				c = line.replace('\n','').replace(' ','').split(":")
#				dict_data[13][files_name]=int(c[1])
#			if "Combined pairs" in line:
#				c = line.replace('\n','').replace(' ','').split(":")
#				dict_data[14][files_name]=int(c[1])

#	path="logs/logs_coverage/"+files_name+"_coverage*"
#	files_cov=glob.glob(path)
#	with open(files_cov[0],'r') as cov:
#		for line in cov:
#			if "Mapped" in line:
#				c = line.split(":")
#				dict_data[16][files_name]=int(c[1])
#				line = next(cov)
#				c = line.split(" ")
#				dict_data[17][files_name]=int(c[0])
#				line = next(cov)
#				c = line.split(":")
#				dict_data[18][files_name]=int(c[1])
#
#	with open("logs/logs_contaminent/Stats_contaminent_"+files_name+".txt",'r') as conta:
#		for line in conta:
#			if "Host_pair_R1" in line:
#				c = line.replace('\n','').split(":")
#				R1_h=c[1]
#				line = next(conta)
#				c = line.replace('\n','').split(":")
#				R2_h=c[1]
#				line = next(conta)
#				c = line.replace('\n','').split(":")
#				wi_h=c[1]
#				line = next(conta)
#				c = line.replace('\n','').split(":")
#				R1_b=c[1]
#				line = next(conta)
#				c = line.replace('\n','').split(":")
#				R2_b=c[1]
#				line = next(conta)
#				c = line.replace('\n','').split(":")
#				wi_b=c[1]
#				dict_data[15][files_name]=int(wi_b)
#				dict_data[10][files_name]=int(R1_h)+int(R2_h)+int(wi_h)
#				dict_data[12][files_name]=int(R1_b)+int(R2_b)+int(wi_b)
#				dict_data[9][files_name]=(int(dict_data[4][files_name])+int(dict_data[5][files_name]))-int(dict_data[10][files_name])
#				dict_data[11][files_name]=dict_data[10][files_name]-dict_data[12][files_name]
##WriteDictToCSV("test.step1.csv",csv_columns,dict_data)
#print("1")
#df_s = pd.read_csv(stats_seq, delimiter='\t')
#df_l=pd.read_csv(lineage, delimiter=',')
#df_c = pd.read_csv(count_table, delimiter=',')
#df_s_l=df_s.merge(df_l, on='tax_id', how='left')
#df=df_s_l.merge(df_c, on='qseqid', how='left')
#df = df[df.iloc[: , -1].notna()]
#tmp_df=df.iloc[:, np.r_[0,30:len(df. columns)]]
#tmp_df.to_csv(r'tmp_df.csv')
#df.to_csv(r'df.csv')
#n=0
#for files_name in list_files :
#	n+=1
#	for i in (0,1,4,5,16,18):
#		dict_data[i]['All-sample']+=int(dict_data[i][files_name])
#	for i in (2,3,6,7,17):
#		if dict_data[i]['All-sample']==0:
#			dict_data[i]['All-sample']=round((float(dict_data[i][files_name])), 2)
#		else:
#			dict_data[i]['All-sample']= round((float(dict_data[i]['All-sample']+float(dict_data[i][files_name]))/2), 2)
#	stat_vir=df[files_name] != 0
#	df1=(df[stat_vir].family.value_counts())
#	dict_data[31][files_name]=df1.count()
#	df1_1=(df.family.value_counts())
#	dict_data[31]['All-sample']=df1_1.count()
#	df2=(df[stat_vir].genus.value_counts())
#	dict_data[32][files_name]=df2.count()
#	df2_2=(df.genus.value_counts())
#	dict_data[32]['All-sample']=df2_2.count()
#	df3=(df[stat_vir].species.value_counts())
#	dict_data[33][files_name]=df3.count()
#	df3_3=(df.species.value_counts())
#	dict_data[33]['All-sample']=df3_3.count()
#	raws = df[[files_name]]
#	sum0=sum10=sum100=sum1000=sum10000=sum0c=0
#	numcont=0
#	for index, raw in raws.iterrows():
#		numcont+=1
#		if raw[files_name] != 0 :
#			sum0+=raw[files_name]
#			print()
#		if 10 <= raw[files_name] < 100 :
#			sum10+=1
#		if 100 <= raw[files_name] <1000 :
#			sum100+=1
#		if 1000 <= raw[files_name] < 10000 :
#			sum1000+=1
#		if raw[files_name] >= 10000 :
#			sum10000+=1
#	dict_data[26][files_name]=sum0
#	dict_data[27][files_name]=sum10
#	dict_data[28][files_name]=sum100
#	dict_data[29][files_name]=sum1000
#	dict_data[30][files_name]=sum10000
##	df = pd.read_csv(stats_seq, delimiter=',')
#	raws = df[[files_name,'qseqid']]
#	in_contigs=0
#	numcont=0
#	for index, raw in raws.iterrows():
#		numcont+=1
#		if raw[files_name] != 0 :
#			if raw['qseqid'].startswith('k') or raw['qseqid'].startswith('C'):
#				in_contigs+=raw[files_name]
#	out_oncitgs=sum0-in_contigs
#	dict_data[23]['All-sample']=numcont
#	dict_data[24][files_name]=in_contigs

#	dict_data[25][files_name]=out_oncitgs
#	for i in range(19,24):
#		dict_data[i][files_name]='-'
#	for i in range(24,27):
#		dict_data[i]['All-sample']+=dict_data[i][files_name]
#print("2")

#tot_hit=tot_hit10=tot_hit100=tot_hit1000=tot_hit10000=0


#for index, raw in tmp_df.iterrows():
#	sum_raw=0
#	for input_files in path_files :
#		if input_files.endswith(ext1+ext_file):
#			files_name=input_files.split(ext1)[0]
#			sum_raw+=raw[files_name]
#			tot_hit+=1
#	if 10 <= sum_raw< 100 :
#		tot_hit10+=1
#	if 100 <= sum_raw < 1000 :
#		tot_hit100+=1
#	if 1000 <= sum_raw < 10000 :
#		tot_hit1000+=1
#	if sum_raw >= 10000 :
#		tot_hit10000+=1
#dict_data[27]['All-sample']=tot_hit10
#dict_data[27]['All-sample']=tot_hit10
#dict_data[28]['All-sample']=tot_hit100
#dict_data[29]['All-sample']=tot_hit1000
#dict_data[30]['All-sample']=tot_hit10000
#print("3")
#path=os.listdir("logs/logsAssembly/")
#for input_files in path :
#	if input_files.endswith(run+"_assembly_stats.txt"):
#		with open("logs/logsAssembly/"+input_files,'r') as asb:
#			data=pd.read_csv(asb, sep="\t", header=None)
#			dict_data[19]['All-sample']=len(data.index)
#			c=data.min()
#			dict_data[20]['All-sample']=c.values[1]
#			c=data.max()
#			dict_data[21]['All-sample']=c.values[1]
#			c=data.mean()
#			dict_data[22]['All-sample']="%.2f" %(c.values[0])
#
#csv_file = out
#WriteDictToCSV(csv_file,csv_columns,dict_data)



#!/usr/bin/env python
"""Resynchronize 2 fastq or fastq.gz files (R1 and R2) after they have been
trimmed and cleaned
WARNING! This program assumes that the fastq file uses EXACTLY four lines per
    sequence
Three output files are generated. The first two files contain the reads of the
    pairs that match and the third contains the solitary reads.
Usage:
    python fastqCombinePairedEnd.py input1 input2 separator
input1 = LEFT  fastq or fastq.gz file (R1)
input2 = RIGHT fastq or fastq.gz file (R2)
separator = character that separates the name of the read from the part that
    describes if it goes on the left or right, usually with characters '1' or
    '2'.  The separator is often a space, but could be another character. A
    space is used by default. If the sequence names do not contain two parts
    and you want to use the full name info to pair your sequences, use 'None'
    (as text) for the separator. Eg:
        python fastqCombinePairedEnd.py input1 input2 None
"""

# Importing modules
import gzip
import sys

# Parsing user input
try:
    in1 = sys.argv[1]
    in2 = sys.argv[2]
    out_1 = sys.argv[3]
    out_2 = sys.argv[4]
    out_S = sys.argv[5]

except:
    print(__doc__)
    sys.exit(1)

separator = " "

# Defining classes
class Fastq(object):
    """Fastq object with name and sequence
    """

    def __init__(self, name, seq, name2, qual):
        self.name = name
        self.seq = seq
        self.name2 = name2
        self.qual = qual

    def getShortname(self, separator):
        if separator:
            self.temp = self.name.split(separator)
            del(self.temp[-1])
            return separator.join(self.temp)
        else:
            return self.name

    def write_to_file(self, handle):
        handle.write(self.name + "\n")
        handle.write(self.seq + "\n")
        handle.write(self.name2 + "\n")
        handle.write(self.qual + "\n")

# Defining functions
def myopen(infile, mode="rt"):
    if infile.endswith(".gz"):
        return gzip.open(infile, mode=mode)
    else:
        return open(infile, mode=mode)

def fastq_parser(infile):
    """Takes a fastq file infile and returns a fastq object iterator
    """
    
    with myopen(infile) as f:
        while True:
            name = f.readline().strip()
            if not name:
                break

            seq = f.readline().strip()
            name2 = f.readline().strip()
            qual = f.readline().strip()
            yield Fastq(name, seq, name2, qual)

# Main
if __name__ == "__main__":
    seq1_dict = {}
    seq2_dict = {}
    seq1 = fastq_parser(in1)
    seq2 = fastq_parser(in2)
    s1_finished = False
    s2_finished = False

    if in1.endswith('.gz'): 
    	outSuffix='.fastq.gz'
    else:
    	outSuffix='.fastq'
    	
    with myopen(out_1, "wt") as out1:
        with myopen(out_2, "wt") as out2:
            with myopen(out_S, "wt") as out3:
                while not (s1_finished and s2_finished):
                    try:
                        s1 = next(seq1)
                    except:
                        s1_finished = True
                    try:
                        s2 = next(seq2)
                    except:
                        s2_finished = True

                    # Add new sequences to hashes
                    if not s1_finished:
                        seq1_dict[s1.getShortname(separator)] = s1
                    if not s2_finished:
                        seq2_dict[s2.getShortname(separator)] = s2

                    if not s1_finished and s1.getShortname(separator) in seq2_dict:
                        seq1_dict[s1.getShortname(separator)].write_to_file(out1)
                        seq1_dict.pop(s1.getShortname(separator))
                        seq2_dict[s1.getShortname(separator)].write_to_file(out2)
                        seq2_dict.pop(s1.getShortname(separator))

                    if not s2_finished and s2.getShortname(separator) in seq1_dict:
                        seq2_dict[s2.getShortname(separator)].write_to_file(out2)
                        seq2_dict.pop(s2.getShortname(separator))
                        seq1_dict[s2.getShortname(separator)].write_to_file(out1)
                        seq1_dict.pop(s2.getShortname(separator))
                        
                # Treat all unpaired reads
                for r in seq1_dict.values():
                    r.write_to_file(out3)

                for r in seq2_dict.values():
                    r.write_to_file(out3)



import sys, os
from collections import Counter
import pandas as pd
import csv
from os.path import exists
import glob
import numpy as np

Path_file_mat= sys.argv[1]
file_mat= os.listdir(Path_file_mat)
output_full= sys.argv[2]

count_table = pd.DataFrame({'qseqid' : []})
for files in file_mat:
	print(files)
	if "contig" in files:
		df=pd.read_csv(Path_file_mat+files, sep=' ').drop_duplicates()
		count_table=count_table.merge(df, on='qseqid', how='outer').fillna(0)
count_table.iloc[: , 1:] = count_table.iloc[: , 1:].astype(int)

count_table.to_csv(sys.argv[2],index=False)



import sys, os
from collections import Counter
import pandas as pd
import csv

#Take 3 input Blast resum , Mat files , lineage
#Output two files Coverage  of contigs for each samples and coverage for lineages.

#Files with the blast infos for each seq/contigs. Output of the rule Join_seq_acc_taxo
Seq_hit_info_file= sys.argv[1]
with open(Seq_hit_info_file,'r') as f:
	seq_hit_info = f.readlines()[1:]
#seq_hit_info = open(Seq_hit_info_file, 'r')

# Path to the directory CountsMapping_all/. Outputs of the rule Quantify_contigs_coverage
Path_file_mat= sys.argv[2]
file_mat= os.listdir(Path_file_mat)

Path_file_mat_ext=sys.argv[3]

# File with the taxonomic lineage for each tax_ids. Output of the rule get_lineage_from_taxids
Taxonomy_file=sys.argv[4]
Taxonomy=open(Taxonomy_file, 'r')


output_full= sys.argv[5]


Dic_count={}
for files in file_mat:
	if files.endswith(Path_file_mat_ext):
		f=open(Path_file_mat+files, 'r')
		files_name=files.split(Path_file_mat_ext)[0]
		for line in f :
			line=line.replace('\n','').split()
			print(line)
			if files_name not in Dic_count and files_name!="" :
				Dic_count[files_name]={}
				Dic_count[files_name].update({line[1]:line[0]})
			else:
				Dic_count[files_name].update({line[1]:line[0]})
data_frame = pd.DataFrame(Dic_count).fillna(0)
del Dic_count
del line
f.close()



Dic_count_by_samples={}
data_frame_to_str=data_frame.to_csv()
list_count=data_frame_to_str.split("\n")
samples=list_count[0].split(",")

for line in list_count[1:]:
	liste_count=line.split(',')
	print(liste_count)
	Dic_count_by_samples[liste_count[0]]=[]
	for count in liste_count[1:]:
		Dic_count_by_samples[liste_count[0]].append(int(count))
del list_count
del data_frame_to_str


### Produce a dic with Tax_ids as key and list of contig/seq corresponding as value
Dic_best_hit={}
for hits in seq_hit_info:
	list_hit=hits.replace('\n','').split(',')
	print(list_hit)
	list_hit[0]=list_hit[0].split('/', 1)[0]
	if list_hit[-1] not in Dic_best_hit :
		Dic_best_hit[list_hit[-1]]={list_hit[0]:list_hit[1:-1]}
	else :
		Dic_best_hit[list_hit[-1]].update({list_hit[0]:list_hit[1:-1]})

print(Dic_best_hit)
Dic_count_taxids={}
Sum_count_per_taxids={}
for tax_ids in Dic_best_hit:
	l=[]
	Dic_count_taxids[tax_ids]=[]
	for ids in Dic_best_hit[tax_ids]:
		ids=ids.split('/', 1)[0] #Remove the /1 or /2 from the ids of sequences
		try:
			Dic_count_taxids[tax_ids].append(Dic_count_by_samples[ids])
		except KeyError:
			continue
	for listes in Dic_count_taxids[tax_ids]:
		l.append(listes)
		Sum_count_per_taxids[tax_ids]=list(map(sum, zip(*l)))

#tax_hits_infos={}
#for tax_ids in Dic_best_hit:
#	tax_hits_infos[tax_ids]=[]
#	num_contigs=0
#	total_query_length_cont=0
#	total_subject_length=0
#	total_align_length_cont=0
#	ratio_length=0
#	percentage_id=0
#	percent_query=0
#	percent_subject=0
#	match_length=0
#	list_subject=[]
#	number_sigleton=0
#	total_align_length_sin=0
#	percentage_id_sigleton=0
#	contig_id=[]
#	average_p_id=0
#	for seqs in Dic_best_hit[tax_ids]:
#		if seqs.startswith("k") or seqs.startswith("C") :
#			Dic_best_hit[tax_ids][seqs]
#			num_contigs+=1
#			total_align_length_cont+=int(Dic_best_hit[tax_ids][seqs][3])
#			total_query_length_cont+=int(Dic_best_hit[tax_ids][seqs][1])
#			total_subject_length+=int(Dic_best_hit[tax_ids][seqs][2])
#			percent_query+=float(Dic_best_hit[tax_ids][seqs][8])*((int(Dic_best_hit[tax_ids][seqs][1])))
#			percent_subject+=((float(Dic_best_hit[tax_ids][seqs][3])*100)/int(Dic_best_hit[tax_ids][seqs][2]))*((int(Dic_best_hit[tax_ids][seqs][2])))
#			percentage_id+=float(Dic_best_hit[tax_ids][seqs][9])*(float(Dic_best_hit[tax_ids][seqs][3]))
#			match_length+=(float(Dic_best_hit[tax_ids][seqs][3]))
#			average_p_id+=(float(Dic_best_hit[tax_ids][seqs][9]))
#			contig_id.append(float(Dic_best_hit[tax_ids][seqs][9]))
#			if Dic_best_hit[tax_ids][seqs][0] not in list_subject:
#				list_subject.append(Dic_best_hit[tax_ids][seqs][0])
#			else:
#				continue
#		else:
#			number_sigleton+=1
#			total_align_length_sin+=int(Dic_best_hit[tax_ids][seqs][3])
#			if number_sigleton!=0:
#				percentage_id_sigleton+=float(Dic_best_hit[tax_ids][seqs][9])*((int(Dic_best_hit[tax_ids][seqs][3])))
#			else:
#				percentage_id_sigleton=0
#	if num_contigs !=0 and number_sigleton!=0 :
#		tax_hits_infos[tax_ids].extend(( "%.2f" % float(match_length/num_contigs), num_contigs, number_sigleton, "%.2f" % float(total_query_length_cont/num_contigs), total_query_length_cont, "%.2f" % ((percent_query/(int(total_query_length_cont)))), len(list_subject), "%.2f" % ((percent_subject/(int(total_subject_length)))), "%.2f" % ((percentage_id/(int(total_align_length_cont)))), min(contig_id) , max(contig_id) , "%.2f" % (average_p_id/num_contigs),
#		"%.2f" % ((percentage_id_sigleton/(int(total_align_length_sin)))/number_sigleton)))
#	if num_contigs !=0 and number_sigleton==0 :
#		tax_hits_infos[tax_ids].extend(("%.2f" % float(match_length/num_contigs), num_contigs, number_sigleton, "%.2f" % float(total_query_length_cont/num_contigs), total_query_length_cont, "%.2f" % ((percent_query/(int(total_query_length_cont)))/num_contigs), len(list_subject), "%.2f" % ((percent_subject/(int(total_subject_length)))), "%.2f" % ((percentage_id/(int(total_align_length_cont)))),min(contig_id) , max(contig_id) , "%.2f" % (average_p_id/num_contigs), "-"))
#	elif num_contigs ==0 and number_sigleton !=0 :
#		tax_hits_infos[tax_ids].extend(( "-", "-" , number_sigleton, "-", "-","-", "-", "-", "-","-","-","-", "%.2f" % ((percentage_id_sigleton/(int(total_align_length_sin))))))
#	tax_hits_infos[tax_ids]=tax_hits_infos[tax_ids]+Sum_count_per_taxids[tax_ids][:]

ranks=Taxonomy.readline().replace('\n','').split("|")
#stat_info=["Avg_match_length","n_Contigs","n_Singleton","Avg_contigs_length","Sum_contigs_length","Avg_Coverage_contigs","n_subject","Avg_Coverage_subject","Pond_Avg_contigs_p_id","min_contigs_p_id","max_contig_p_id", "avg_contig_p_id","Avg_singleton_p_id" ]
#lineage_coverage=[ranks+stat_info+samples[1:]]

stat_info_brut=["qseqid"]
lineage_brut=[stat_info_brut+samples[1:]]
dic_lineage={}
#for lineage in Taxonomy:
#	list_lineage=lineage.replace('\n','').split('|')
#	lineage_coverage.append(list_lineage + tax_hits_infos[list_lineage[0]])
#	dic_lineage[list_lineage[0]]=list_lineage

list_table=[]
DIc_test={}
for tax_id in Dic_best_hit:
	for key in Dic_best_hit[tax_id].keys():
		key=key.split('/', 1)[0]
		try:
			lineage_brut.append([key]+list_table + Dic_count_by_samples[key])
		except KeyError:
			continue
#with open(output_stat, "w") as f:
#    writer = csv.writer(f)
#    writer.writerows(lineage_coverage)
with open(output_full, "w") as f1:
    writer = csv.writer(f1)
    
    writer.writerows(lineage_brut)
Taxonomy.close()



#!/usr/bin/env python3

# Module import
import rich_click as click
from pathlib import Path
import subprocess
import sys
# import pyyaml module
import yaml


def verif_config(config):
    '''
    This function check each variable give by user in config.yaml
    '''
    ### Open the file and load the file (return a dict) ###
    try:  # Check if config is at yaml format.
        with open(config) as f:
            data = yaml.safe_load(f)
    except:  # If not at yaml format return a error.
        click.secho(f"ERROR: you'r config file {config} is not at Yaml format, please check it.", fg='red', bold=True, err=True)
        exit()

    ### Check fastq directory ###
    # Check if is a directory
    directory_fastq = Path(data['fastq'])
    if not directory_fastq.is_dir():
        click.secho(f"ERROR: You'r fastq directory {directory_fastq} is not a directory, please check it.", fg='red', bold=True, err=True)
        exit()
    # Check if they have fastq (with ext_R1 and ext) and if number R1 = number R2
    nb_R1 = len(sorted(directory_fastq.rglob(f'*{data["ext_R1"]}{data["ext"]}')))
    nb_R2 = len(sorted(directory_fastq.rglob(f'*{data["ext_R2"]}{data["ext"]}')))
    if nb_R1 == 0:
        click.secho(f"ERROR: You'r fastq directory {directory_fastq} doesn't contains fasta with '{data['ext_R1']}{data['ext']}' or '{data['ext_R2']}{data['ext']}' extension",
            fg='red', bold=True, err=True)
        exit()
    elif nb_R1 != nb_R2:
        click.secho(
            f"ERROR: You'r fastq directory {directory_fastq} doesn't two reads file per sample, please check it",
            fg='red', bold=True, err=True)
        exit()

    ### Check path_diamond ###
    # Check if file exist ( name* )
    diamond_nr = Path(data['base_nr'])
    diamond_directory = diamond_nr.parent
    name_database = diamond_nr.name
    # Count file from diamond database give by user
    nb_file = len(sorted(diamond_directory.rglob(f'{name_database}*')))
    # Check if they have at least 1 file in database
    if nb_file == 0:
        click.secho(
            f"ERROR: You'r diamond nr database '{diamond_nr}' doesn't exist, please check you're path",
            fg='red', bold=True, err=True)
        exit()

    ### Check blast nr path ###
    blast_nt = Path(data['base_nt'])
    blast_directory = blast_nt.parent
    name_database = blast_nt.name
    # Count file from diamond database give by user
    nb_file = len(sorted(blast_directory.rglob(f'{name_database}*')))
    # Check if they have at least 8 file in database ( 8 extension of makeblastdn) here name* , for name.00, name .01 of makeblastdb output
    if nb_file < 8:
        raise click.secho(
            f"ERROR: You'r blast nt database '{blast_nt}' doesn't exist, please check you're path",
            fg='red', bold=True, err=True)
    silva_db = Path(data['rRNA_bact'])
    silva_directory = silva_db.parent
    nb_file = len(sorted(blast_directory.rglob(f'{name_database}*')))
    if nb_file < 18:
        raise click.secho(
            f"ERROR: You'r silva database '{silva_db}' doesn't exist, please check you're path or re-install it with snakevir install_cluster.",
            fg='red', bold=True, err=True)

def __run(config, add_option):
    """
    Run the snbakevir workflow.
    """
    path_snakevir = Path(__file__).resolve().parent.parent
    verif_config(config)
    add_option = " ".join(add_option)
    cmd = f"snakemake  -s {path_snakevir}/snakefile --configfile {config} --profile {path_snakevir}/install_files/profile/slurm --cluster-config {path_snakevir}/install_files/cluster.yaml --show-failed-logs {add_option}"
    process = subprocess.run(cmd, shell=True, check=False, stdout=sys.stdout, stderr=sys.stderr)
    if int(process.returncode) >= 1:
        raise

if __name__ == '__main__':
    __run()



#!/usr/bin/env python3

# Module import
from pathlib import Path

def __make_config(name, fastq, r1, r2, ext, path_diamond_nr, path_blast_nt, a3, a5, output, host_genome,output_directory):
    """
    The command make_config is used for create config fime at yaml format for snakevir. You have 2 choice, you can use arguement
    for write all information needed in config or you can only use some argument (-o is mandatory) and wirte in the file after
    the missing information.
    """
    # Path to install file (directory which contain the default config file
    install_path = f'{Path(__file__).resolve().parent.parent}/install_files'
    new_config = list()
    with open(f'{install_path}/config.yaml', 'r') as config_file:
        for line in config_file:
            if line.startswith("run:"):
                line = f"run: {name}\n"
            if line.startswith("fastq:"):
                line = f"fastq: {fastq}\n"
            if line.startswith("host_genome:"):
                line = f"host_genome: {host_genome}\n"
            if line.startswith("output:"):
                line = f"output: {output_directory}\n"
            if line.startswith("ext_R1:"):
                line = f"ext_R1: {r1}\n"
            if line.startswith("ext_R2:"):
                line = f"ext_R2: {r2}\n"
            if line.startswith("ext:"):
                line = f"ext: {ext}\n"
            if line.startswith("base_nr:"):
                line = f"base_nr: {path_diamond_nr}\n"
            if line.startswith("base_nt:"):
                line = f"base_nt: {path_blast_nt}\n"
            if line.startswith("A3:" ):
                line = f"A3: {a3}\n"
            if line.startswith("A5:"):
                line = f"A5: {a5}\n"
            new_config.append(line)

    with open(f'{output}', 'w') as new_file:
        new_file.write("".join(new_config))



import sys, os
from ete3 import NCBITaxa
from collections import Counter
import pandas as pd
import csv
import subprocess
import pickle
from Bio import Entrez
import time

Entrez.email = "exbrayat.antoni@gmail.com"
Entrez.api_key = "2ce25cc7a838f78f0e1cf4ff91b9c035b309"
Taxonomy_file=sys.argv[1]
seq_hit_file=sys.argv[2]
Host_taxo_file=sys.argv[3]
output= sys.argv[4]

Taxonomy=open(Taxonomy_file, 'r')
seq_hit=open(seq_hit_file, 'r')
host_taxo=open(Host_taxo_file, 'r')

host_lineage=[]
host_dict={}


for lineage in host_taxo:
	host_lineage=(lineage.replace('\n','').split(','))

	if host_lineage[0] != "virus_tax_id":
		if host_lineage[0] not in host_dict:
			host_dict[host_lineage[0]]=[host_lineage[1:2]]
		else :
			host_dict[host_lineage[0]].append(host_lineage[1:2])

tax_ids=[]
otu_tax={}
taxo_dic=pd.read_csv(sys.argv[1], index_col=0).to_dict()


for lineage in Taxonomy:
	rank=lineage.replace('\n','').replace('\r','').split(",")
	if rank[0]!="tax_id":
		if rank[0] not in tax_ids:
			tax_ids.append(rank[0])
		if rank[0] not in otu_tax and  rank[0]!="tax_id": 
			otu_tax[rank[0]]={"hosts_taxon":[],"hosts_gb":[],"hosts_tax_id":[],"hosts_superkingdom":[],"hosts_kingdom":[],
						"hosts_phylum":[],"hosts_class":[],"hosts_order":[],"hosts_family":[],"hosts_subfamily":[],"hosts_genus":[],"hosts_species":[]}
						
seq_tax={}
for seq in seq_hit:
	seq=seq.replace('\n','').replace('\r','').split(",")
	if seq[-1] in tax_ids:
		if seq[1] not in seq_tax: 
			seq_tax[seq[1]]={"tax_id":seq[-1],"hosts_taxon":"","hosts_gb":[],"hosts_tax_id":[],"hosts_superkingdom":"","hosts_kingdom":"",
						"hosts_phylum":"","hosts_class":"","hosts_order":"","hosts_family":"","hosts_subfamily":"","hosts_genus":"","hosts_species":""}

ncbi = NCBITaxa()
liste_host=[]
out=[]
n=0
host_all=[]
dic={}


for sequence in seq_tax:
	seq=seq_tax[sequence]["tax_id"]
	if seq in host_dict:
		host_id=[]
		superkingdom=[]
		clas=[]
		order=[]
		family=[]
		subfamily=[]
		genus=[]
		species=[]
		for hosts in host_dict[seq]:
			if  hosts[0]!='' and hosts[0] not in host_id:
				host_id.append(hosts[0])
		if host_id !='':
			seq_tax[sequence]["hosts_tax_id"]=host_id
	print(n,"/",len(seq_tax))
	n+=1
	ranks2lineage={}
	liste=[]
	fieldnames=[]
	fieldnames1=[]
	host_list=[]
	host_id="NA"
	taxid="txid"+ str(seq)
	handle = Entrez.esearch(db="protein", retmax=20, term= str(taxid), idtype="acc")
	record = Entrez.read(handle)
	sseqids=record["IdList"]
	if sequence in sseqids:
		sseqids.remove(sequence.replace('"',''))
	sseqids.insert(0, sequence.replace('"',''))
	cleaned_host_gb=""
	source=""
	for ids in sseqids:
		handle = Entrez.efetch(db="protein", id=str(ids), retmode="xml")
		records = Entrez.read(handle)
		records=records[0]['GBSeq_feature-table'][0]['GBFeature_quals']
		for record in records:
			if "host" in record.values(): 
				cleaned_host_gb=record['GBQualifier_value']
				species =  cleaned_host_gb.replace(" ", "+").strip()
#				time.sleep(1)
				search = Entrez.esearch(term=species, db="taxonomy", retmode="xml")
				org_txi = Entrez.read(search)
				print("host ", cleaned_host_gb)
				if not org_txi['IdList']:
					continue
				else:
					host_id = org_txi['IdList'][0]
			elif "host" not in record.values() and "isolation_source" in record.values():
				source=record['GBQualifier_value']
		if source !="":
			break		
		if cleaned_host_gb !="":
			break
	if cleaned_host_gb=='' and source !="": 
		cleaned_host_gb=source

	if cleaned_host_gb!='':
		seq_tax[sequence]["hosts_gb"].append(cleaned_host_gb)
		liste.extend([sequence , cleaned_host_gb])
		liste_host.append(liste)
		
	if host_id not in seq_tax[sequence]["hosts_tax_id"] and host_id!="NA":
		seq_tax[sequence]["hosts_tax_id"].append(host_id)
	for hosts_ids in seq_tax[sequence]["hosts_tax_id"]:
		if hosts_ids != "" and len(seq_tax[sequence]["hosts_tax_id"])>0:
			try:
				lineage=ncbi.get_lineage(hosts_ids)
			except:
				continue
			lineage2name = ncbi.translate_to_names(lineage)
			lineage2ranks=ncbi.get_rank(lineage)
			Dic_lineage2name=dict(zip(lineage, lineage2name))
			for (taxid,rank) in lineage2ranks.items():
				if taxid in Dic_lineage2name:
					if sequence not in dic:
						dic[sequence]={}
					if rank not in dic[sequence] :
						dic[sequence][rank]=[Dic_lineage2name[taxid]]
					elif Dic_lineage2name[taxid] not in dic[sequence][rank]:
						dic[sequence][rank].append(Dic_lineage2name[taxid])
			if sequence in dic:
				if dic[sequence].get('superkingdom')!=None:
					seq_tax[sequence]["hosts_superkingdom"]=(dic[sequence].get('superkingdom'))
				if dic[sequence].get('kingdom')!=None:
					seq_tax[sequence]["hosts_kingdom"]=(dic[sequence].get('kingdom'))
				if dic[sequence].get('phylum')!=None:
					seq_tax[sequence]["hosts_phylum"]=(dic[sequence].get('phylum'))
				if dic[sequence].get('class')!=None:
					seq_tax[sequence]["hosts_class"]=(dic[sequence].get('class'))
				if dic[sequence].get('order')!=None:
					seq_tax[sequence]["hosts_order"]=(dic[sequence].get('order'))
				if dic[sequence].get('family')!=None:
					seq_tax[sequence]["hosts_family"]=(dic[sequence].get('family'))
				if dic[sequence].get('subfamily')!=None:
					seq_tax[sequence]["hosts_subfamily"]=(dic[sequence].get('subfamily'))
				if dic[sequence].get('genus')!=None:
					seq_tax[sequence]["hosts_genus"]=(dic[sequence].get('genus'))
				if dic[sequence].get('species')!=None:
					seq_tax[sequence]["hosts_species"]=(dic[sequence].get('species'))
					
#a_file = open("host.pkl", "rb")
#seq_tax = pickle.load(a_file)



a_file = open("host2.pkl", "wb")
pickle.dump(seq_tax, a_file)
a_file.close()

###a_file = open("host.pkl", "rb")
###output = pickle.load(a_file)
###print(output)


for key in seq_tax:
	if "Bacteria" in str(seq_tax[key]["hosts_superkingdom"])and not seq_tax[key]["hosts_taxon"]:
		seq_tax[key]["hosts_taxon"]=["Bacteria"]
#	if "Arthropoda"  in seq_tax[key]["hosts_phylum"] and "Chordata"  in seq_tax[key]["hosts_phylum"]and not seq_tax[key]["hosts_taxon"]:
#		seq_tax[key]["hosts_taxon"]=["Arbovirus"]
	elif len(seq_tax[key]["hosts_class"])==1 :
		if "Mammalia" in str(seq_tax[key]["hosts_class"])and not seq_tax[key]["hosts_taxon"]:
			seq_tax[key]["hosts_taxon"]=["Vertebrate"]
		elif "Aves" in str(seq_tax[key]["hosts_class"])and not seq_tax[key]["hosts_taxon"]:
			seq_tax[key]["hosts_taxon"]=["Vertebrate"]
		elif "Arthropoda" in str(seq_tax[key]["hosts_phylum"]):

			if "Insecta" in str(seq_tax[key]["hosts_class"]):
				if "Diptera" in str(seq_tax[key]["hosts_order"]) and  not seq_tax[key]["hosts_taxon"]:
					seq_tax[key]["hosts_taxon"]=["Diptera"]
				else:
					seq_tax[key]["hosts_taxon"]=["Insecta_other"]
			else:
				seq_tax[key]["hosts_taxon"]=["Arthropoda_other"]
		elif "Viridiplantae" in str(seq_tax[key]["hosts_kingdom"]):
			seq_tax[key]["hosts_taxon"]=["Viridiplantae"]
		elif "Fungi" in str(seq_tax[key]["hosts_kingdom"]):
			seq_tax[key]["hosts_taxon"]=["Fungi"]

	elif len(seq_tax[key]["hosts_class"])>1 :
		if "Mammalia" in str(seq_tax[key]["hosts_class"])and not seq_tax[key]["hosts_taxon"]:
			seq_tax[key]["hosts_taxon"]=["Vertebrate"]
		elif "Aves" in str(seq_tax[key]["hosts_class"]) and  not seq_tax[key]["hosts_taxon"]:
			seq_tax[key]["hosts_taxon"]=["Vertebrate"]
		elif "Arthropoda" in str(seq_tax[key]["hosts_phylum"]):
			if "Insecta" in str(seq_tax[key]["hosts_class"]):
				if "Diptera" in str(seq_tax[key]["hosts_order"]) and  not not seq_tax[key]["hosts_taxon"]:
					seq_tax[key]["hosts_taxon"]=["Diptera"]
				else:
					seq_tax[key]["hosts_taxon"]=["Insecta_other"]
			elif not seq_tax[key]["hosts_taxon"]:
				seq_tax[key]["hosts_taxon"]=["Arthropoda_other"]
		elif "Viridiplantae" in str(seq_tax[key]["hosts_kingdom"])  and not seq_tax[key]["hosts_taxon"]:
			seq_tax[key]["hosts_taxon"]=["Viridiplantae"]
		elif  not seq_tax[key]["hosts_taxon"] and seq_tax[key]["hosts_tax_id"][0]!="NA" :
			seq_tax[key]["hosts_taxon"]=["Other"]
			
	elif len(seq_tax[key]["hosts_tax_id"])>=1 and not seq_tax[key]["hosts_taxon"] and seq_tax[key]["hosts_tax_id"][0]!="NA":
		seq_tax[key]["hosts_taxon"]=["Other"]
		
	elif not seq_tax[key]["hosts_taxon"] and  "species" in taxo_dic:
		if "phage" in taxo_dic["species"][int(seq_tax[key]["tax_id"])]:
			seq_tax[key]["hosts_taxon"]=["Bacteria"]


			
	for key1, value in seq_tax[key].items():
		tax_id_m=str(seq_tax[key]["tax_id"])
		if key1 !="tax_id":
			otu_tax[tax_id_m][key1]=list(set(otu_tax[tax_id_m][key1]) | set(seq_tax[key][key1]))
		if isinstance(value, list):
			seq_tax[key].update({key1:";".join(value)})

for key in otu_tax:
	for key1, value in otu_tax[key].items():
		if isinstance(value, list):
			otu_tax[key].update({key1:";".join(value)})


#df=pd.DataFrame.from_dict(seq_tax,orient='index')
#df.index.name = 'sseqid'
#df.reset_index(inplace=True)
#column_names=["sseqid","tax_id","hosts_taxon","hosts_gb","hosts_tax_id","hosts_superkingdom","hosts_kingdom","hosts_phylum","hosts_class","hosts_order","hosts_family","hosts_subfamily","hosts_genus","hosts_species"]
#df = df.reindex(columns=column_names)
#df.replace("", "NA", inplace=True)
#df.to_csv(r'contig_host_test.csv', sep=',',index=False)

df1=pd.DataFrame.from_dict(otu_tax,orient='index')
df1.index.name = 'tax_id'
df1.reset_index(inplace=True)
#column_names=["tax_id","hosts_taxon","hosts_gb","hosts_tax_id","hosts_superkingdom","hosts_kingdom","hosts_phylum","hosts_class","hosts_order","hosts_family","hosts_subfamily","hosts_genus","hosts_species"]
#df1 =df1.reindex(columns=column_names)
df1.replace("", "NA", inplace=True)
df1.to_csv(sys.argv[4], sep=',',index=False,header=False)




#!/usr/bin/env python3

# Module import
import rich_click as click
from pathlib import Path
import subprocess
from collections import defaultdict
# import pyyaml module
import yaml

def __edit_cluster(partition, account, edit):
    """
    The command make_config is used for create config fime at yaml format for snakevir. You have 2 choice, you can use arguement
    for write all information needed in config or you can only use some argument (-o is mandatory) and wirte in the file after
    the missing information.
    """
    # Path to install file (directory which contain the default config file
    install_path = f'{Path(__file__).resolve().parent.parent}/install_files'
    # Change partition & account names
    new_cluster = list()
    with open(f'{install_path}/cluster.yaml', 'r') as cluster_file:
        for line in cluster_file:
            if line.strip().startswith("partition:"):
                if partition == "False": # Check if partition is give by user or not
                    partition_default = line.strip().split(':')[-1].strip() # Retrieve partition from config if partition is not provide by user
                else :
                    old_partition = line.strip().split(':')[-1].strip()
                    line = line.replace(old_partition,partition)
            if line.strip().startswith("account:") :
                if account == "False": # Check if account is give by user or not
                    account_default = line.strip().split(':')[-1].strip() # Retrieve account from config if account is not provide by user
                else :
                    old_account = line.strip().split(':')[-1].strip()
                    line = line.replace(old_account,account)
            new_cluster.append(line)

    with open(f'{install_path}/cluster.yaml', 'w') as new_file:
        new_file.write("".join(new_cluster))

    # Open editor to modify ressources
    if edit:
        click.edit(require_save=True, extension='.yaml', filename=f'{install_path}/cluster.yaml')

    # Check parititon & account in config file after editing
    partition_default = defaultdict(list) # For check all parition for each rules.
    account_default = defaultdict(list) # For check all account for each rules.

    with open(f'{install_path}/cluster.yaml') as f:
        data = yaml.safe_load(f)
    for rule in data:
        if "partition" in data[rule]:
            partition_rule = data[rule]["partition"]
            partition_default[partition_rule].append(rule)
    for rule in data:
        if "account" in data[rule]:
            account_rule = data[rule]["account"]
            account_default[account_rule].append(rule)

    # Check account (with groups shell command)
    available_account = subprocess.check_output("groups", shell=True).decode("utf8").strip().split()
    for account in account_default:
        if account not in available_account:
            txt_rule = ", ".join(partition_default[account])
            raise click.secho(
                f"ERROR: You'r account '{account_default}' for {txt_rule} doesn't exist, please check you're account.",
                fg='red', bold=True, err=True)

    # Check partition (with sinfo shell command)
    available_partition = subprocess.check_output(r"""sinfo -s | cut -d" " -f1""", shell=True).decode("utf8").strip().replace('*','').split("\n")[1:]
    for partition in partition_default:
        if partition not in available_partition:
            txt_rule = ", ".join(partition_default[partition])
            raise click.secho(
                f"ERROR: You'r partition '{partition}' for rule '{txt_rule}' doesn't exist in this cluster , please check the partition available with sinfo command.",
                fg='red', bold=True, err=True)



import sys, os
from collections import Counter
import pandas as pd
import csv
from os.path import exists
import glob
import numpy as np
# import pandas as pd
# import locale
# locale.setlocale(locale.LC_ALL, '')
path_files=os.listdir(sys.argv[1])
ext1=sys.argv[2]
ext2=sys.argv[3]
ext_file=sys.argv[4]
run=sys.argv[5]
stats_seq=sys.argv[6]
lineage=sys.argv[7]
count_table=sys.argv[8]
out=sys.argv[9]

def WriteDictToCSV(csv_file,csv_columns,dict_data):
    with open(csv_file, 'w') as csvfile:
        writer = csv.DictWriter(csvfile, fieldnames=csv_columns)
        writer.writeheader()
        for data in dict_data:
            writer.writerow(data)
            
            
csv_columns=['Stats','All-sample']
dict_data = [{'Stats':"Total read R1"},{'Stats':"Total read R2"},{'Stats':"Avg read length R1"},{'Stats':"Avg read length R2"},{'Stats':"Reads 1 cleaned"},{'Stats':"Reads 2 cleaned"},
{'Stats':"Avg cleaned read length R1 (after cutadapt)"},{'Stats':"Avg cleaned read length R2 (after cutadapt)"},{'Stats':"Avg insert_size"},{'Stats':"Map on Diptera"},{'Stats':"UnMapped on Diptera"},{'Stats':"Map on bacteria"},{'Stats':"UnMapped on bacteria"},
{'Stats':"Total pairs"},{'Stats':"Combined pairs"},{'Stats':"Widows"},{'Stats':"Assembled"},{'Stats':"Reads spread across n contig (nb for each samples and Avg for all_samples)"},{'Stats':"Singletons"},{'Stats':"Nb total of contigs"},
{'Stats':"Min contigs length"},{'Stats':"Max contigs length"},{'Stats':"Avg contigs length"},{'Stats':"Contigs with viral hit"},{'Stats':"Nb of reads with viral hit 'inside' a contig"},{'Stats':"Nb of eads with viral hit as singleton"},{'Stats':"Reads with viral hit"},
{'Stats':"Nb of viral hit 10 =< Reads < 100 "},{'Stats':"Nb of viral hit 100 =< Reads < 1000"},{'Stats':"Nb of viral hit 1000 =< Reads < 10000"},{'Stats':"Nb of viral hit 10000=< Reads"},{'Stats':"Nb viral family"},{'Stats':"Nb viral genus"},{'Stats':"Nb viral species"}]

for i in range(27):
	dict_data[i]['All-sample']=int(0)
list_files=[]
for input_files in path_files :
	if input_files.endswith(ext1+ext_file):
		file_name=input_files.split(ext1+ext_file)[0]
		csv_columns.append(file_name)
		list_files.append(file_name)

for files_name in list_files :
	with open("logs/logscutadapt/"+files_name+ext1+"_cut1.log",'r') as cut1_1:
		for line in cut1_1:
			if "Total reads processed" in line:
				c = line.replace(',','').replace(' ','').replace('\n','').split(":")
				dict_data[0][files_name]=int(c[-1])
			if "Total written" in line:
				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
				dict_data[2][files_name]="%.2f" % (int(c1[1])/int(c[-1]))

	with open("logs/logscutadapt/"+files_name+ext2+"_cut1.log",'r') as cut1_2:
		for line in cut1_2:
			if "Total reads processed" in line:
				c = line.replace(',','').replace(' ','').replace('\n','').split(":")
				dict_data[1][files_name]=int(c[-1])
			if "Total written" in line:
				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
				dict_data[3][files_name]="%.2f" % (int(c1[1])/int(c[-1]))
	with open("logs/logscutadapt/"+files_name+ext1+"_cut2.log",'r') as cut2_1:
		for line in cut2_1:
			if "Reads written" in line:
				c = line.replace(',','').replace('(',':').replace('\n','').replace(' ','').split(":")
				dict_data[4][files_name]=int(c[2])
			if "Total written" in line:
			
				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
				dict_data[6][files_name]="%.2f" % (int(c1[1])/int(c[2]))
	with open("logs/logscutadapt/"+files_name+ext2+"_cut2.log",'r') as cut2_2:
		for line in cut2_2:
			if "Reads written" in line:
				c = line.replace(',','').replace('(',':').replace('\n','').replace(' ','').split(":")
				dict_data[5][files_name]=int(c[2])
			if "Total written" in line:
				c1 = line.replace(',','').replace('b',':').replace('\n','').replace(' ','').split(":")
				dict_data[7][files_name]="%.2f" % (int(c1[1])/int(c[2]))

	path="logs/insert_size/"+files_name+"_insert_size_metrics_"+run+".txt"
	files_insert=glob.glob(path)
	with open(files_insert[0],'r') as insert:
#	with open("logs/insert_size/"+files_name+"_insert_size_metrics.txt",'r') as insert:
		for line in insert:
			if "LIBRARY	READ_GROUP" in line:
				line = next(insert)
				c = line.split("\t")
				dict_data[8][files_name]="%.2f" % (float(c[4]))

	with open("logs/logsFLASH/"+files_name+"_flash.log",'r') as flash:
		for line in flash:
			if "Total pairs" in line:
				c = line.replace('\n','').replace(' ','').split(":")
				dict_data[13][files_name]=int(c[1])
			if "Combined pairs" in line:
				c = line.replace('\n','').replace(' ','').split(":")
				dict_data[14][files_name]=int(c[1])

	path="logs/logs_coverage/"+files_name+"_coverage*"
	files_cov=glob.glob(path)
	with open(files_cov[0],'r') as cov:
		for line in cov:
			if "Mapped" in line:
				c = line.split(":")
				dict_data[16][files_name]=int(c[1])
				line = next(cov)
				c = line.split(" ")
				dict_data[17][files_name]=int(c[0])
				line = next(cov)
				c = line.split(":")
				dict_data[18][files_name]=int(c[1])
				
	with open("logs/logs_contaminent/Stats_contaminent_"+files_name+".txt",'r') as conta:
		for line in conta:
			if "Host_pair_R1" in line:
				c = line.replace('\n','').split(":")
				R1_h=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				R2_h=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				wi_h=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				R1_b=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				R2_b=c[1]
				line = next(conta)
				c = line.replace('\n','').split(":")
				wi_b=c[1]
				dict_data[15][files_name]=int(wi_b)
				dict_data[10][files_name]=int(R1_h)+int(R2_h)+int(wi_h)
				dict_data[12][files_name]=int(R1_b)+int(R2_b)+int(wi_b)
				dict_data[9][files_name]=(int(dict_data[4][files_name])+int(dict_data[5][files_name]))-int(dict_data[10][files_name])
				dict_data[11][files_name]=dict_data[10][files_name]-dict_data[12][files_name]
#WriteDictToCSV("test.step1.csv",csv_columns,dict_data)
print("1")
df_s = pd.read_csv(stats_seq, delimiter='\t')
df_l=pd.read_csv(lineage, delimiter=',')
df_c = pd.read_csv(count_table, delimiter=',')
df_s_l=df_s.merge(df_l, on='tax_id', how='left')
df=df_s_l.merge(df_c, on='qseqid', how='left')
df = df[df.iloc[: , -1].notna()]
tmp_df=df.iloc[:, np.r_[0,30:len(df. columns)]]
tmp_df.to_csv(r'tmp_df.csv')
df.to_csv(r'df.csv')
n=0
for files_name in list_files :
	n+=1
	for i in (0,1,4,5,16,18):
		dict_data[i]['All-sample']+=int(dict_data[i][files_name])
	for i in (2,3,6,7,17):
		if dict_data[i]['All-sample']==0:
			dict_data[i]['All-sample']=round((float(dict_data[i][files_name])), 2)
		else:
			dict_data[i]['All-sample']= round((float(dict_data[i]['All-sample']+float(dict_data[i][files_name]))/2), 2)
	stat_vir=df[files_name] != 0
	df1=(df[stat_vir].family.value_counts())
	dict_data[31][files_name]=df1.count()
	df1_1=(df.family.value_counts())
	dict_data[31]['All-sample']=df1_1.count()
	df2=(df[stat_vir].genus.value_counts())
	dict_data[32][files_name]=df2.count()
	df2_2=(df.genus.value_counts())
	dict_data[32]['All-sample']=df2_2.count()
	df3=(df[stat_vir].species.value_counts())
	dict_data[33][files_name]=df3.count()
	df3_3=(df.species.value_counts())
	dict_data[33]['All-sample']=df3_3.count()
	raws = df[[files_name]]
	sum0=sum10=sum100=sum1000=sum10000=sum0c=0
	numcont=0
	for index, raw in raws.iterrows():
		numcont+=1
		if raw[files_name] != 0 :
			sum0+=raw[files_name]
			print()
		if 10 <= raw[files_name] < 100 :
			sum10+=1
		if 100 <= raw[files_name] <1000 :
			sum100+=1
		if 1000 <= raw[files_name] < 10000 :
			sum1000+=1
		if raw[files_name] >= 10000 :
			sum10000+=1
	dict_data[26][files_name]=sum0
	dict_data[27][files_name]=sum10
	dict_data[28][files_name]=sum100
	dict_data[29][files_name]=sum1000
	dict_data[30][files_name]=sum10000
#	df = pd.read_csv(stats_seq, delimiter=',')
	raws = df[[files_name,'qseqid']]
	in_contigs=0
	numcont=0
	for index, raw in raws.iterrows():
		numcont+=1
		if raw[files_name] != 0 :
			if raw['qseqid'].startswith('k') or raw['qseqid'].startswith('C'):
				in_contigs+=raw[files_name]
	out_oncitgs=sum0-in_contigs
	dict_data[23]['All-sample']=numcont
	dict_data[24][files_name]=in_contigs

	dict_data[25][files_name]=out_oncitgs
	for i in range(19,24):
		dict_data[i][files_name]='-'
	for i in range(24,27):
		dict_data[i]['All-sample']+=dict_data[i][files_name]
print("2")

tot_hit=tot_hit10=tot_hit100=tot_hit1000=tot_hit10000=0


for index, raw in tmp_df.iterrows():
	sum_raw=0
	for input_files in path_files :
		if input_files.endswith(ext1+ext_file):
			files_name=input_files.split(ext1)[0]
			sum_raw+=raw[files_name]
			tot_hit+=1
	if 10 <= sum_raw< 100 :
		tot_hit10+=1
	if 100 <= sum_raw < 1000 :
		tot_hit100+=1
	if 1000 <= sum_raw < 10000 :
		tot_hit1000+=1
	if sum_raw >= 10000 :
		tot_hit10000+=1
dict_data[27]['All-sample']=tot_hit10
dict_data[27]['All-sample']=tot_hit10
dict_data[28]['All-sample']=tot_hit100
dict_data[29]['All-sample']=tot_hit1000
dict_data[30]['All-sample']=tot_hit10000
print("3")
path=os.listdir("logs/logsAssembly/")
for input_files in path :
	if input_files.endswith(run+"_assembly_stats.txt"):
		with open("logs/logsAssembly/"+input_files,'r') as asb:
			data=pd.read_csv(asb, sep="\t", header=None)
			dict_data[19]['All-sample']=len(data.index)
			c=data.min()
			dict_data[20]['All-sample']=c.values[1]
			c=data.max()
			dict_data[21]['All-sample']=c.values[1]
			c=data.mean()
			dict_data[22]['All-sample']="%.2f" %(c.values[0])
			
csv_file = out
WriteDictToCSV(csv_file,csv_columns,dict_data)



import sys, os
from ete3 import NCBITaxa
from collections import Counter
import pandas as pd
import csv
import subprocess
import numpy as np
from collections import Counter

blast_contig_hit_file=sys.argv[1]
blast_contig_tax_file=sys.argv[2]
tax_table_file=sys.argv[3]
out_file=sys.argv[4]
#Taxonomy_full_file=sys.argv[2]
#Ref_taxonomy_file=sys.argv[3]
#ICTV_taxo_file=sys.argv[4]
#Host_taxo_file=sys.argv[5]
#output_stat= sys.argv[6]
#output_seq= sys.argv[7]

if not os.path.exists('tmp'):
	os.makedirs('tmp')

blast_contig_hit=pd.read_csv(blast_contig_hit_file, sep=',')
print(blast_contig_hit)
blast_contig_hit=blast_contig_hit.fillna("none")
print("1")

blast_contig_tax=pd.read_csv(blast_contig_tax_file, sep=',')
blast_contig_tax=blast_contig_tax.fillna("none")

tax_table=pd.read_csv(tax_table_file, sep=',')
tax_table=tax_table[tax_table.superkingdom == "Viruses"]
tax_table=tax_table.fillna("NA")


start_table=blast_contig_hit.merge(blast_contig_tax, on="tax_id", how='left')

start_table=start_table.fillna("none")
out_table=start_table.drop_duplicates(subset=['qseqid'])
out_table=out_table[out_table.species != "none"]
start_table=start_table[start_table['qseqid'].isin(out_table['qseqid'])]
out_table=out_table.drop_duplicates(subset=['tax_id'])



out_table.to_csv(r'out_table.csv', sep=',')
start_table.to_csv(r'start_table.csv', sep=',')

def Average(lst):
    return sum(lst) / len(lst)
    
def GetKey(dictA,val):
   for key, value in dictA.items():
      if val == value:
         return key
      continue
print("2")
qseq_dic={}
tax_dic={}
pident={}
n=0
for qseqid_0 in  start_table["qseqid"].unique():
	n+=1

	if qseqid_0 not in qseq_dic:
		if start_table[start_table ["qseqid"]== qseqid_0]["species"].tolist()[0]!="none":
			qseq_dic[qseqid_0]=start_table[start_table ["qseqid"]== qseqid_0]["family"].tolist()
			qseq_dic[qseqid_0].insert(0,start_table[start_table ["qseqid"]== qseqid_0]["tax_id"].tolist()[0])
print("ok1")
for qseqid_0 in  start_table["qseqid"].unique():
	taxid_0=start_table[start_table ["qseqid"]== qseqid_0]["tax_id"].tolist()[0]
	if taxid_0 not in pident:
		pident[taxid_0]={}
	for index, row in start_table[start_table ["qseqid"]== qseqid_0].iterrows():
		if row["family"] not in pident[taxid_0]:
			pident[taxid_0][row["family"]]=[row["pident"]]
		else:
			pident[taxid_0][row["family"]].append(row["pident"])

for k, v in pident.items():
	for k1 in pident[k]:
		if "none" not in pident[k][k1]:
			pident[k][k1]=round(Average(pident[k][k1]),2)
print("ok2")
df=pd.DataFrame.from_dict(qseq_dic,orient='index')
df=df.rename(columns={0: "tax_id"})
df.fillna("none")
df1 = df.groupby("tax_id").agg(lambda x: x.tolist())
df1.fillna("none")

df1['combined']=df1[1]+df1[2]+df1[3]+df1[4]+df1[5]
print("ok3")
df1.fillna("none")
df1.to_csv(r'df1.csv', sep=',')
for index, row in df1.iterrows():
	row.combined=["none" if v is None else v for v in row.combined]
	if index not in tax_dic:
		tax_dic[index]=[len(row.combined)/5]
	if len(set(row.combined))==1:
		tax_dic[index].extend([len(set(row.combined)),dict(Counter(row.combined))])
	else:
		row.combined[:]=(value for value in row.combined if value != "none")
		tax_dic[index].extend([len(set(row.combined)),dict(Counter(row.combined))])
	if tax_dic[index][2]:
		tax_dic[index].extend([sum(list(tax_dic[index][2].values())),max(list(tax_dic[index][2].values()))])
		tax_dic[index].append(pident[index])
		try :
			tax_dic[index].append(max({k: tax_dic[index][5][k] for k in tax_dic[index][5].keys() if k not in ('none')}, key={k: tax_dic[index][5][k] for k in tax_dic[index][5].keys() if k not in ('none')}.get))
		except ValueError:
			tax_dic[index].append(max( tax_dic[index][5], key=tax_dic[index][5].get))
		try :
			tax_dic[index].append(max(list({k: tax_dic[index][5][k] for k in tax_dic[index][5].keys() if k not in ('none')}.values())))
		except ValueError:
			tax_dic[index].append(max(list(tax_dic[index][5].values())))
		tax_dic[index].append(max(tax_dic[index][2].values())*100/sum(list(tax_dic[index][2].values())))
		if max(tax_dic[index][2], key=tax_dic[index][2].get)==tax_dic[index][6]:
			if max(list(tax_dic[index][2].values()))*100/sum(list(tax_dic[index][2].values())) >= 75 :
					tax_dic[index].append(max(tax_dic[index][2], key=tax_dic[index][2].get))
			elif tax_dic[index][1]==3 and max(list(tax_dic[index][2].values()))*100/sum(list(tax_dic[index][2].values())) >= 60:
				tax_dic[index].append(max(tax_dic[index][2], key=tax_dic[index][2].get))
			elif tax_dic[index][1]==2 and max(list(tax_dic[index][2].values()))*100/sum(list(tax_dic[index][2].values())) > 50:
				tax_dic[index].append(max(tax_dic[index][2], key=tax_dic[index][2].get))

print("ok4")
for k, v in tax_dic.items():
	if v[1] == 0:
		del tax_dic[k]



df2=pd.DataFrame.from_dict(tax_dic,orient='index')
df2new = df2[[9]]
df2new=df2new.fillna("NA")
df2new.index.name = 'tax_id'

#tax_table.set_index('tax_id').join(df2new.set_index('tax_id'))
df2new.reset_index(inplace=True)

tax_table['tax_id'] = tax_table['tax_id'].astype(int)

tax_table=tax_table.merge(df2new, on="tax_id", how='left')

mask = (tax_table['family'] == "NA") & (tax_table[9] != "NA") & (tax_table[9] != "none")
tax_table['family'][mask] = tax_table[9][mask]
df2.columns=['n_contigs','n_species','fam_out','n_out','max_out','fam_pid','best_predict_fam','max_pid','per','final_fam']

df3 = df2[['fam_out','fam_pid','final_fam']]
print(df3)
df3.index.name = 'tax_id'
df3.reset_index(inplace=True)
print("ok5")

tax_table=tax_table.merge(df3, on="tax_id", how='left')

#data['column2'][mask] = 3

#if tax_table['family']=="" :
##	print(tax_table['family'])
#def fx(x):
#    if tax_table['family']=="NA" and tax_table[9]!="none" and tax_table[9]!="NA":
#        return tax_table[9]
#    else:
#        return tax_table['family']

#tax_table['family']=tax_table.apply(lambda x : fx(x),axis=1)
del tax_table[9]

#tax_table.to_csv(r'tax_table.csv', sep='\t', index=False)
tax_table.to_csv(sys.argv[4], sep='\t', index=False, mode = 'w')
comp_inf = sys.argv[1].replace(sys.argv[1].split('/')[-1],'comp_inf.csv')
df3.to_csv(comp_inf, sep='\t', index=False, mode = 'w')
#tax_table.to_csv(r out_file, sep=',')
##df.to_csv(r'qseq_dic.csv', sep=',')
##df1.to_csv(r'qseq_dic_taxid.csv', sep=',')
#df2.to_csv(r'df2.csv', sep=',')

##		tax_dic[index].extend([sum(tax_dic[index][2].values()),max(tax_dic[index][2].values()),max(tax_dic[index][2], key=tax_dic[index][2].get)])
#	max_key = max(tax_dic[index][2], key=tax_dic[index][2].get)

#	print(index, len(set(row.combined)),dict(Counter(row.combined)))

#df2 = df.groupby("tax_id").agg(lambda x: ','.join(x).split(','))
#df1=df1.merge(df2, on="tax_id", how='left')
#df['combined']= df.to_str(values).tolist()
#print(df1.groupby(0))

#df2.to_csv(r'tax_dic.csv', sep=',')
#qseq_dic.to_csv("qseq_dic.csv")
#		qseq_dic[qseqid_0]=[blast_contig_hit.loc[blast_contig_hit["qseqid"]== qseqid_0]["qseqid"].count(),blast_contig_hit[blast_contig_hit ["qseqid"]== qseqid_0]["family"].tolist()]
#	n+=1
#	print(n,blast_contig_hit[blast_contig_hit["qseqid"]== qseqid]["qseqid"])
#	print(qseqid,blast_contig_hit.loc[blast_contig_hit["qseqid"]== qseqid].counts(),blast_contig_hit[blast_contig_hit ["qseqid"]== qseqid]["tax_id"].nunique())
#	print(qseqid,blast_contig_hit.loc[blast_contig_hit["qseqid"]== qseqid]["qseqid"].count(),blast_contig_hit[blast_contig_hit ["qseqid"]== qseqid]["family"].nunique())
#	df1=blast_contig_hit[blast_contig_hit ["qseqid"]== qseqid]["tax_id"].value_counts()
#for hits in blast_contig_hit:
#
#	ref_lineage.append(lineage.replace('\n','').split(','))
#print(blast_contig_hit)


#ref_lineage=[]
#ICTV_lineage=[]
#host_lineage=[]
#host_dict={}
#Taxonomy=open(Taxonomy_file, 'r')
#Taxonomy_full=open(Taxonomy_full_file, 'r')
#Ref_taxonomy=open(Ref_taxonomy_file, 'r')
#ICTV_taxo=open(ICTV_taxo_file, 'r')
#host_taxo=open(Host_taxo_file, 'r')

#for lineage in Ref_taxonomy:
#	ref_lineage.append(lineage.replace('\n','').split(','))


#for lineage in host_taxo:
#	host_lineage=(lineage.replace('\n','').split(','))
#	if host_lineage[0] != "virus_tax_id":
#		if host_lineage[0] not in host_dict:
#			host_dict[host_lineage[0]]=[host_lineage[1:]]
#		else:
#			host_dict[host_lineage[0]].append(host_lineage[1:])
#for lineage in ICTV_taxo:
#	ICTV_lineage.append(lineage.replace('\n','').split(','))

#liste_full=[]
#liste_fullseqid=[]

#for lineage in Taxonomy_full:
#	rank=lineage.replace('\n','').replace('\r','').split(",")
#	if rank[0]=="qseqid":
#		ranks_full=rank[0:16]+["clade"]+rank[16:21]+["cluster"]+["Nucleic_acid"]+rank[21:22]+["hosts_gb"]+["hosts_tax_id"]+["hosts_superkingdom"]+["hosts_class"]+["hosts_order"]+["hosts_family"]+["hosts_subfamily"]+["hosts_genus"]+["hosts_species"]+rank[22:]
#	elif rank[14]=="Viruses":
#		ranks_full=rank[0:16]+["NA"]+rank[16:21]+["NA"]+["NA"]+rank[21:22]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+rank[22:]
#	else:
#		continue
#	liste_full.append(ranks_full)

#host_all=[]
#for ranks_full in liste_full:
#	index=liste_full.index(ranks_full)
#	if liste_full[index][14]=="Viruses":
#		for ictv_rank in ICTV_lineage:
#			if liste_full[index][21]==ictv_rank[3] and  liste_full[index][21]!="NA" :
#				if liste_full[index][20]=="NA":
#					liste_full[index][20]=ictv_rank[2]
#				if liste_full[index][18]=="NA":
#					liste_full[index][18]=ictv_rank[1]
#				if liste_full[index][17]=="NA":
#					liste_full[index][17]=ictv_rank[0]
#				if liste_full[index][23]=="NA":
#					liste_full[index][23]=ictv_rank[4]

#		for ref_rank1 in ref_lineage:

#			if liste_full[index][21]==ref_rank1[6] and  liste_full[index][21]!="NA" :
#				if liste_full[index][20]=="NA":
#					liste_full[index][20]=ref_rank1[3]
#				if liste_full[index][18]=="NA":
#					liste_full[index][18]=ref_rank1[2]
#				if liste_full[index][17]=="NA":
#					liste_full[index][17]=ref_rank1[1]
#				if liste_full[index][23]=="NA":
#					liste_full[index][23]=ref_rank1[5]
#				if liste_full[index][22]=="NA":
#					liste_full[index][22]=ref_rank1[4]
#				if liste_full[index][16]=="NA":
#					liste_full[index][16]=ref_rank1[0]

#		if liste_full[index][18]!="NA" and liste_full[index][22]=="NA":
#			liste_full[index][22]=liste_full[index][18]
#		if liste_full[index][19]!="NA" and liste_full[index][22]=="NA":
#			liste_full[index][22]=liste_full[index][16]
#		if liste_full[index][16]!="NA" and liste_full[index][22]=="NA":
#			liste_full[index][22]=liste_full[index][17]

#		if liste_full[index][23]=="NA":
#			if "ssRNA negative-strand viruses" in liste_full[index][24]:
#				liste_full[index][23]="ssRNA(-)"
#			if "ssRNA positive-strand viruses" in liste_full[index][24]:
#				liste_full[index][23]="ssRNA(+)"
#			if "dsRNA viruses" in liste_full[index][24]:
#				liste_full[index][23]="dsRNA"
#			if "RNA satellites" in liste_full[index][24]:
#				liste_full[index][23]="RNA satellites"
#			if "ssDNA viruses" in liste_full[index][24]:
#				liste_full[index][23]="ssDNA(+/-)"
#			if "dsDNA viruses" in liste_full[index][24]:
#				liste_full[index][23]="dsDNA"
#			if "unclassified ssRNA viruses" in liste_full[index][24]:
#				liste_full[index][23]="ssRNA"
#			if "unclassified RNA viruses" in liste_full[index][24]:
#				 liste_full[index][23]="RNA"
#		if  liste_full[index][1] not in seqid :
#			seqid.append(liste_full[index][1])
#		if ranks_full[13] in host_dict:
#			print(ranks_full[13])
#			host_id=""
#			superkingdom=[]
#			clas=[]
#			order=[]
#			family=[]
#			subfamily=[]
#			genus=[]
#			species=[]
#			for hosts in host_dict[ranks_full[13] ]:
#				if host_id=="" and hosts[0]!='':
#					host_id+=str(hosts[0]).replace("[","").replace("]","")
#				else:
#					host_id+="|"+str(hosts[0])
#				if hosts[1] not in superkingdom:
#					superkingdom.append(hosts[1])
#				if hosts[2] not in clas:
#					clas.append(hosts[2])
#				if hosts[3] not in order:
#					order.append(hosts[3])
#				if hosts[4] not in family:
#					family.append(hosts[4])
#				if hosts[5] not in subfamily:
#					subfamily.append(hosts[5])
#				if hosts[6] not in genus:
#					genus.append(hosts[6])
#				if hosts[7] not in species:
#					species.append(hosts[7])
#			liste_full[index][26]=(str(host_id).replace("']","").replace("['",""))
#			liste_full[index][27]=(str(superkingdom).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_full[index][28]=(str(clas).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_full[index][29]=(str(order).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_full[index][30]=(str(family).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_full[index][31]=(str(subfamily).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_full[index][32]=(str(genus).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_full[index][33]=(str(species).replace("]","").replace("[","").replace("'","").replace(", ","|"))


#ncbi = NCBITaxa()
#liste_host=[]
#stat_by_seq=[]
#n=0

#print("ok2")
#desired_ranks = ['superkingdom', 'class', 'order', 'family', 'subfamily','genus','species']
#for ids in seqid:
#	print(n,"/",len(seqid))
#	n+=1
#	ranks2lineage={}
#	liste=[]
#	fieldnames=[]
#	fieldnames1=[]
#	host_list=[]
#	host_id="NA"
##	command = 'esearch -db protein -query '+ str(ids) +'| efetch -format gb | grep -oP "/host=\K.*"'
##	proc = subprocess.Popen(command,stdout=subprocess.PIPE,shell=True)
##	(out, err) = proc.communicate()
##	host_gb=out.decode("utf-8").rstrip().replace('"','').replace("[u'","")
##	cleaned_host_gb=host_gb.split(' (', 1)[0]
#	cleaned_host_gb=''
#	if cleaned_host_gb=="mosquito" or cleaned_host_gb=="mosquitoes":
#		cleaned_host_gb="Culicoidea"
#	if cleaned_host_gb!='':
#		liste.extend([ids , cleaned_host_gb])
#		liste_host.append(liste)
#		name2taxid = ncbi.get_name_translator([cleaned_host_gb])
#		fieldnames.append(host_gb)
#		if cleaned_host_gb in name2taxid:
#			host_id=int(name2taxid[cleaned_host_gb][0])
#			lineage = ncbi.get_lineage(host_id)
#			fieldnames.append(host_id)
#	if cleaned_host_gb!='' and cleaned_host_gb in name2taxid:
#		lineage2name = ncbi.translate_to_names(lineage)
#		lineage2ranks=ncbi.get_rank(lineage)
#		Dic_lineage2name=dict(zip(lineage, lineage2name))
#		for (taxid,rank) in lineage2ranks.items():
#			if rank in desired_ranks:
#				if not rank in ranks2lineage:
#					ranks2lineage[rank]=[taxid]
#				else:
#					ranks2lineage[rank].append(taxid)
#		ranks2names={}
#		for rank in ranks2lineage:
#			ranks2names[rank]=""
#			for i in range(len(ranks2lineage[rank])):
#				ranks2names[rank]+=Dic_lineage2name[ranks2lineage[rank][i]]+";"
#			ranks2names[rank]=ranks2names[rank][:-1]
#		ranks2names['host_id']=host_id
#		for rank in desired_ranks:
#			fieldnames1.append(ranks2names.get(rank, 'NA'))
#	host_list=[ids]+fieldnames+fieldnames1
#	host_all.append(host_list)


#print("ok3")
#dict_host={}
#for seq in liste_full:
#	if seq[0] == "qseqid":
#		stat_by_seq.append(seq[0:])
#	else:
#		for host in host_all:
#			l=[]
#			if seq[1]==host[0]:
#				if len(host)==2:
#					l=seq[0:25]+host[1:]+seq[26:]
#				elif len(host)>2:
#					l=seq[0:25]+host[1:]+seq[34:]
#				else:
#					l=seq[0:]
#				stat_by_seq.append(l)
#				if len(host)>=2:
#					for idx_host, val_host in enumerate(host):
#						if  isinstance(val_host,str):
#							val_host=val_host
#						if seq[13] not in dict_host :
#							dict_host[seq[13]]={idx_host:[val_host]}
#						elif idx_host not in dict_host[seq[13]]:
#							dict_host[seq[13]].update({idx_host:[val_host]})
#						elif  val_host!='NA' and val_host not in dict_host[seq[13]][idx_host]:
#							dict_host[seq[13]][idx_host].append(val_host)


#for ranks_full in liste_full:
#	index=liste_full.index(ranks_full)
#	if str(liste_full[index][13]) in dict_host and len(dict_host[str(liste_full[index][13])])>2:
#		liste_full[index][25]=str(dict_host[str(liste_full[index][13])][2]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#		liste_full[index][26]=str(dict_host[str(liste_full[index][13])][3]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#		liste_full[index][27]=str(dict_host[str(liste_full[index][13])][4]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#		liste_full[index][28]=str(dict_host[str(liste_full[index][13])][5]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#		liste_full[index][29]=str(dict_host[str(liste_full[index][13])][6]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#		liste_full[index][30]=str(dict_host[str(liste_full[index][13])][7]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#		liste_full[index][31]=str(dict_host[str(liste_full[index][13])][8]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#		liste_full[index][32]=str(dict_host[str(liste_full[index][13])][9]).replace("]","").replace("[","").replace("'","").replace(", ","|")

#liste_species=[]
#for lineage in Taxonomy:
#	rank=lineage.replace('\n','').replace('\r','').split(",")
#	if rank[0]=="tax_id":
#		ranks=rank[0:3]+["clade"]+rank[3:8]+["cluster"]+["Nucleic_acid"]+["hosts_gb"]+["hosts_tax_id"]+["hosts_superkingdom"]+["hosts_class"]+["hosts_order"]+["hosts_family"]+["hosts_subfamily"]+["hosts_genus"]+["hosts_species"]+rank[9:]
#	elif rank[1]=="Viruses":
#		ranks=rank[0:3]+["NA"]+rank[3:8]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+["NA"]+rank[9:]
#	else:
#		continue
#	liste_species.append(ranks)


#for ranks in liste_species:
#	index=liste_species.index(ranks)
#	if liste_species[index][1]=="Viruses":
#		for ictv_rank in ICTV_lineage:
#			if liste_species[index][8]==ictv_rank[3] and liste_species[index][8]!="NA":
#				if liste_species[index][7]=="NA":
#					liste_species[index][7]=ictv_rank[2]
#				if liste_species[index][5]=="NA":
#					liste_species[index][5]=ictv_rank[1]
#				if liste_species[index][4]=="NA":
#					liste_species[index][4]=ictv_rank[0]
#				if liste_species[index][10]=="NA":
#					liste_species[index][10]=ictv_rank[4]


#		for ref_rank1 in ref_lineage:
#			if liste_species[index][8]==ref_rank1[6] and liste_species[index][8]!="NA" :
#				if liste_species[index][7]=="NA":
#					liste_species[index][7]=ref_rank1[3]
#				if liste_species[index][5]=="NA":
#					liste_species[index][5]=ref_rank1[2]
#				if liste_species[index][4]=="NA":
#					liste_species[index][4]=ref_rank1[1]
#				if liste_species[index][10]=="NA":
#					liste_species[index][10]=ref_rank1[5]
#				if liste_species[index][9]=="NA":
#					liste_species[index][9]=ref_rank1[4]

#		if liste_species[index][5]!="NA" and liste_species[index][9]=="NA":
#			liste_species[index][9]=liste_species[index][5]
#		if liste_species[index][7]!="NA" and liste_species[index][9]=="NA":
#			liste_species[index][9]=liste_species[index][7]
#		if liste_species[index][4]!="NA" and liste_species[index][9]=="NA":
#			liste_species[index][9]=liste_species[index][4]

#		if liste_species[index][10]=="NA":
#			if "ssRNA negative-strand viruses" in liste_species[index][11]:
#				liste_species[index][10]="ssRNA(-)"
#			if "ssRNA positive-strand viruses" in liste_species[index][11]:
#				liste_species[index][10]="ssRNA(+)"
#			if "dsRNA viruses" in liste_species[index][11]:
#				liste_species[index][10]="dsRNA"
#			if "RNA satellites" in liste_species[index][11]:
#				liste_species[index][10]="RNA satellites"
#			if "ssDNA viruses" in liste_species[index][11]:
#				liste_species[index][10]="ssDNA(+/-)"
#			if "dsDNA viruses" in liste_species[index][11]:
#				liste_species[index][10]="dsDNA"
#			if "unclassified ssRNA viruses" in liste_species[index][11]:
#				liste_species[index][10]="ssRNA"
#			if "unclassified RNA viruses" in liste_species[index][11]:
#				liste_species[index][10]="RNA"
#		if ranks[0] in host_dict:
#			host_id=""
#			superkingdom=[]
#			clas=[]
#			order=[]
#			family=[]
#			subfamily=[]
#			genus=[]
#			species=[]
#			for hosts in host_dict[ranks[0]]:
#				if host_id=="" and hosts[0]!='':
#					host_id+=str(hosts[0]).replace("[","").replace("]","")
#				else:
#					host_id+="|"+str(hosts[0])
#				if hosts[1] not in superkingdom:
#					superkingdom.append(hosts[1])
#				if hosts[2] not in clas:
#					clas.append(hosts[2])
#				if hosts[3] not in order:
#					order.append(hosts[3])
#				if hosts[4] not in family:
#					family.append(hosts[4])
#				if hosts[5] not in subfamily:
#					subfamily.append(hosts[5])
#				if hosts[6] not in genus:
#					genus.append(hosts[6])
#				if hosts[7] not in species:
#					species.append(hosts[7])
#			liste_species[index][12]=(str(host_id).replace("']","").replace("['",""))
#			liste_species[index][13]=(str(superkingdom).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_species[index][14]=(str(clas).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_species[index][15]=(str(order).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_species[index][16]=(str(family).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_species[index][17]=(str(subfamily).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_species[index][18]=(str(genus).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			liste_species[index][19]=(str(species).replace("]","").replace("[","").replace("'","").replace(", ","|"))
#			if str(liste_species[index][0]) in dict_host:
#				liste_species[index][11]=str(dict_host[str(liste_species[index][0])][1]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#		else:
#			if str(liste_species[index][0]) in dict_host and len(dict_host[str(liste_species[index][0])])>2:
#				liste_species[index][12]=str(dict_host[str(liste_species[index][0])][2]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#				liste_species[index][13]=str(dict_host[str(liste_species[index][0])][3]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#				liste_species[index][14]=str(dict_host[str(liste_species[index][0])][4]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#				liste_species[index][15]=str(dict_host[str(liste_species[index][0])][5]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#				liste_species[index][16]=str(dict_host[str(liste_species[index][0])][6]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#				liste_species[index][17]=str(dict_host[str(liste_species[index][0])][7]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#				liste_species[index][18]=str(dict_host[str(liste_species[index][0])][8]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#				liste_species[index][19]=str(dict_host[str(liste_species[index][0])][9]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#			elif str(liste_species[index][0]) in dict_host and len(dict_host[str(liste_species[index][0])])>=2:
#				liste_species[index][11]=str(dict_host[str(liste_species[index][0])][1]).replace("]","").replace("[","").replace("'","").replace(", ","|")
#			else:
#				continue
#list_vir_ids=[]
#dic_host_ids={}
#to_write=[]

#for listes in liste_species:
#	if listes[0]=="tax_id":
#		to_write.append(listes[0:11]+["host_taxon"]+listes[11:])
#	else:
#		to_write.append(listes[0:11]+["NA"]+listes[11:])
#	if "|" in listes[12] :
#		dic_host_ids[listes[0]]=(listes[12].split('|'))
#	else :
#		dic_host_ids[listes[0]]=([listes[12]])

#ranks2lineage={}
#ncbi = NCBITaxa()
#dic={}
#test=[]
#for key in dic_host_ids:
#	for ids in dic_host_ids[key]:
#		if ids != "NA" and ids.isdigit():
#			try:
#				lineage=ncbi.get_lineage(ids)
#			except:
#				pass
#			lineage2name = ncbi.translate_to_names(lineage)
#			lineage2ranks=ncbi.get_rank(lineage)
#			Dic_lineage2name=dict(zip(lineage, lineage2name))
#			for (taxid,rank) in lineage2ranks.items():
#				if taxid in Dic_lineage2name:
#					if key not in dic:
#						dic[key]={}
#					if rank not in dic[key] :
#						dic[key][rank]=[Dic_lineage2name[taxid]]
#					elif Dic_lineage2name[taxid] not in dic[key][rank]:
#						dic[key][rank].append(Dic_lineage2name[taxid])

#	if key in dic:
#		if 'kingdom' in dic[key] and len(dic[key]['kingdom'])==1 and dic[key]['kingdom'][0]!='NA' and dic[key]['kingdom'][0]!='Metazoa':
#			for line in to_write :
#				if key in line:
#					line[11]=str(dic[key]['kingdom'][0])

#		if 'superkingdom' in dic[key] and len(dic[key]['superkingdom'])==1 and dic[key]['superkingdom'][0]!="Eukaryota":
#			for line in to_write:
#				if key in line:
#					line[11]=str(dic[key]['superkingdom'][0])
#		if  'order' in dic[key] and 'class' in dic[key] and len(dic[key]['order'])==1 and len(dic[key]['class'])==1:
#			if dic[key]['order'][0]=='Diptera':
#				if 'family' in dic[key] and dic[key]['family'][0]=='Culicidae':
#					for line in to_write:
#						if key in line:
#							line[11]=str(dic[key]['family'][0])
#				else:
#					for line in to_write:
#						if key in line:
#							line[11]=str(dic[key]['order'][0])
#		for line in to_write:
#			if key in line and  line[11]=='NA' and 'class' in dic[key] and len(dic[key]['class'])==1:
#				if dic[key]['class'][0]=='Insecta':
#							line[11]=str(dic[key]['phylum'][0])
#				elif dic[key]['class'][0]!='Mammalia':
#					if 'Vertebrata' in dic[key]['no rank']:
#								line[11]='Vertebrata'
#				elif dic[key]['class'][0]=='Mammalia':
#							line[11]=str(dic[key]['class'][0])
#			if key in line and  'phylum' in dic[key] and  line[11]=='NA' and dic[key]['phylum'][0]=='Arthropoda':
#						line[11]='Arthropoda'
#			if key in line and  line[11]=='NA':
#				if len(line[13].split('|'))==1:
#					line[11]='Other'


#with open(output_stat, "w") as f:
#    writer = csv.writer(f)
#    writer.writerows(to_write)
#with open(output_seq, "w") as g:
#    writer = csv.writer(g)
#    writer.writerows(stat_by_seq)
#f.close
#g.close



# #!/bin/python

import csv
from ete3 import NCBITaxa
import sys
from collections import Counter

ncbi = NCBITaxa()

def get_all_values(d):
	if isinstance(d, dict):
		for v in d.values():
			yield from get_all_values(v)
	elif isinstance(d, list):
		for v in d:
			yield from get_all_values(v)
	else:
		yield d 

def get_desired_ranks(taxids, desired_ranks):
	try:	
		lineage = ncbi.get_lineage(taxids)
	except:
		lineage = ncbi.get_lineage(1)
	print(taxids,lineage)
	lineage2ranks = ncbi.get_rank(lineage)
	ranks2lineage = {}
	for (taxid,rank) in lineage2ranks.items():
		if not rank in ranks2lineage:
			ranks2lineage[rank]=[taxid]
		else:
			ranks2lineage[rank].append(taxid)
	lineage2name = ncbi.translate_to_names(lineage)
	Dic_lineage2name=dict(zip(lineage, lineage2name))

	ranks2names={}
	for rank in ranks2lineage:
		ranks2names[rank]=""
		for i in range(len(ranks2lineage[rank])):
			ranks2names[rank]+=Dic_lineage2name[ranks2lineage[rank][i]]+";"
		ranks2names[rank]=ranks2names[rank][:-1]
	ranks2names['tax_id']=taxids

	return {'{}'.format(rank): ranks2names.get(rank, 'NA') for rank in desired_ranks}

def main(species, desired_ranks, path):
    with open(path, 'w') as csvfile:
        fieldnames = ['{}'.format(rank) for rank in desired_ranks]
        writer = csv.DictWriter(csvfile, delimiter=',', fieldnames=fieldnames)
        writer.writeheader()
        for taxid in taxids:
            taxid=taxid.replace('\n','')
            writer.writerow(get_desired_ranks(taxid, desired_ranks))
    csvfile.close()


if __name__ == '__main__':
    taxids=sys.stdin.read().split(',')
    desired_ranks = ['tax_id', 'clade','superkingdom','kingdom', 'phylum','class','order', 'family','genus','species', 'no rank']
    path = sys.argv[1]
    main(taxids, desired_ranks, path)



import sys, os
from ete3 import NCBITaxa
from collections import Counter
import pandas as pd
import csv
import subprocess
import numpy as np


file=sys.argv[1]
fileout=sys.argv[2]
cov = pd.read_csv(file)
cov['sum'] = cov.iloc[:, 2:].sum(axis=1)

cov1=cov.groupby("contig").agg(
        min_coverage=pd.NamedAgg(column='sum', aggfunc='min'),
        max_coverage=pd.NamedAgg(column='sum', aggfunc='max'),
        mean_coverage=pd.NamedAgg(column='sum', aggfunc=np.mean),
        median_coverage=pd.NamedAgg(column='sum', aggfunc=np.median)
)

cov1.to_csv(fileout, sep=',', index=True,header=False)



import pickle
import sys, os



with open(sys.argv[1], 'rb') as handle:
    taxo_dict = pickle.load(handle)
    
    
taxo_dict['fam2gent']['Hepadnaviridae']="dsDNA-RT"
taxo_dict['fam2gent']['Caulimoviridae']="dsDNA-RT"

print(taxo_dict['fam2gent']['Caulimoviridae'])

with open('correc_taxo1.pickle', 'wb') as handle:
	pickle.dump(taxo_dict, handle, protocol=pickle.HIGHEST_PROTOCOL)



# #!/bin/python

import sys, os
from Bio import SeqIO
blast_file = sys.argv[1]
fasta_file = sys.argv[2]
output_file = sys.argv[3]



id_blast=[]
with open(blast_file) as f:
    for line in f:
        id_blast.append(line.split()[0])
    id_blast=set(id_blast)
    id_blast=list(id_blast)

fasta_sequences = SeqIO.parse(open(fasta_file),'fasta')
with open(output_file, "w") as f:
    for seq in fasta_sequences:
        if seq.id in id_blast:
            SeqIO.write([seq], f, "fasta")



import sys
import csv
import glob
import pandas as pd
from functools import reduce
import gc
import os, re
import psutil
columns = []
data = {}
ids = set()
df=pd.DataFrame()
nb = 0
for subdir, dirs, files in os.walk(sys.argv[1]):
	for filename in files:
		filepath = subdir + os.sep + filename
		if filename.endswith(".cov") and sys.argv[2] in filename[0:2]:
			nb += 1
			print(filename, nb)
			key = filename.rstrip(".cov")
			with open(filepath) as tsvfile:
				f = pd.read_csv(tsvfile, delimiter="\t", names=["contig", "nuc_pos", key])
				print(f)
				if df.empty:
					df=f
				else:
					df=df.merge(f, how='outer', on=['contig','nuc_pos'])
					print(round(psutil.virtual_memory().used / psutil.virtual_memory().total *100,2))
					df = df.fillna(0).drop_duplicates()
				del f
				gc.collect()

df.to_csv(sys.argv[3], index=False)

def merge_dataframe(subdir, type_file, output):
	"""
	"""
	liste_file = [elt for elt in os.listdir('split_cont/') if type_file in elt]
	liste_panda = [pd.read_csv(f'{subdir}/{file}', delimiter="\t", names=["contig", "nuc_pos", file.rstrip('.cov')]) for file in liste_file]
	df = reduce(lambda df1, df2: pd.merge(df1, df2, how='outer', on=['contig', 'nuc_pos']), liste_panda)
	df = df.fillna(0).drop_duplicates()
	df.to_csv(output, index=False)

# merge_dataframe(sys.argv[1], sys.argv[2], sys.argv[3])

#for subdir, dirs, files in os.walk(sys.argv[1]):
#	for filename in files:
#		filepath = subdir + os.sep + filename
#		if filepath.endswith(".cov"):
#			print(filename)
#			key = filename.rstrip("_on_nova_bat.bam.cov")
#			with open(filepath) as tsvfile:
#				f = pd.read_csv(tsvfile, delimiter="\t", names=["contig", "nuc_pos", key])
#				if df.empty:
#					df=f
#				else:
#					df=df.merge(f, how='outer', on=['contig','nuc_pos'])
#				del f
#				gc.collect()
#df = df.fillna(0).drop_duplicates()

#



#!/usr/bin/env python3

# Module import
import rich_click as click
from pathlib import Path
import sys
import subprocess
from cookiecutter.main import cookiecutter

def create_directory(path):
    '''
    This function create a directory at path given if directory does'nt exist
    '''
    if not Path(f'{path}').exists():
        subprocess.run(f'mkdir {path}', shell=True, check=False, stdout=sys.stdout, stderr=sys.stderr)

def download(file,command, database, skip):
    """
    This function check if file already exist or not and ask at user if re-download is needed.
    """
    value = ""
    # Chek if file already exist
    if Path(file).exists() and not database:
        # Check if we need to skip this function
        if skip:
            return (f"{Path(file).name}")
        # Ask at user if we need to re-download file
        value = click.prompt(f'\nThe file {Path(file).name} already exists, do you want to re-download it ?',
                             type=click.Choice(['y', 'n']))
    # Launch download if file doesn't exist or user want to re-download.
    if not Path(f'{file}').exists() or value == "y" or database:
        subprocess.run(command, shell=True, check=False, stdout=sys.stdout, stderr=sys.stderr)
    else:
        return (f"{Path(file).name}")

def install_conda_env(path, install_path, tool, skip):
    '''
    This function take the path of you're installation (path) and the path of yml file fon conda env (install_path) and
    create the conda environment for snakevir workflow.The force argument allow to install even the conda env is already
    create.
    '''
    # Check if we need to skip this function
    # Path to directory install
    path_conda = f'{path}/snakevir_env'
    # Command for conda installation of snakevire env
    cmd = f'mamba env create --prefix {path_conda} -f {install_path}/snakevir_environment.yml'
    # If conda env already exist, check if we re-install it or no
    if Path(path_conda).exists():
        # Check if we need to skip this function
        if skip:
            click.secho(f'\nThe conda env "{Path(path_conda).stem}" already exist, skip conda installation.\n', fg='green', bold=True)
            return()
        if not tool:
            value = click.prompt(f'\nThe env "{path_conda}" already exists, do you want to re-install it ?',
                             type=click.Choice(['y', 'n']))
        else:
            value = "y"
        if value == 'y':
            command = f'mamba env remove -p {path_conda}'
            subprocess.run(command, shell=True, check=False, stdout=sys.stdout, stderr=sys.stderr)
        # If no, the new command line is nothing.
        else:
            click.secho(f'\nThe conda env "{Path(path_conda).stem}" already exist, skip conda installation.\n',
                        fg='green', bold=True)
            cmd = ''
    # Launch command for install or do nothing
    if cmd != '':
        click.secho(f'\n* Installation of conda environment at "{path_conda}"\n', fg='green', bold=True)

    install_conda = subprocess.run(cmd, shell=True, check=False, stdout=sys.stdout, stderr=sys.stderr)
    if int(install_conda.returncode) != 0: # install_conda.returncode !=0 means that they have error in installation
        click.secho('')
        click.secho(f'Error : They have some problems with conda env installation.', fg='white', bg='red', bold=True)
        click.secho('')
        sys.exit(1)

def install_database(path, database_path, install_path, database, skip):
    '''
     This function take the path of you're installation (path) and the path of the directory install_path and
     install all database needed by snakevir except diamond nr and nt database.
     '''
    # Check if conda env has been already create (we use diamond for upload database)
    if Path(f'{path}/snakevir_env').exists():
        create_directory(database_path)
        # Download accession2taxid
        list_skip = list() # Init list which contain all fils skiped
        ## for nucleotide
        command = f'wget ftp://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/nucl_gb.accession2taxid.gz -O {database_path}nucl_gb.accession2taxid.gz;' \
                  f'gunzip {database_path}nucl_gb.accession2taxid.gz'
        list_skip.append(download(f'{database_path}nucl_gb.accession2taxid',command, database, skip))
        ## for portein
        command = f'wget ftp://ftp.ncbi.nlm.nih.gov/pub/taxonomy/accession2taxid/prot.accession2taxid.gz -O {database_path}prot.accession2taxid.gz;' \
                  f'gunzip {database_path}prot.accession2taxid.gz'
        list_skip.append(download(f'{database_path}prot.accession2taxid', command, database, skip))
        # Donwload virust host database
        command = f'wget https://www.genome.jp/ftp/db/virushostdb/virushostdb.tsv -O {database_path}virushostdb.tsv'
        list_skip.append(download(f'{database_path}virushostdb.tsv', command, database, skip))
        # Prepare silva database
        command = f'echo "\n\t* Extract Archive";' \
                  f'wget https://github.com/FlorianCHA/snakevir/raw/master/snakevir/install_files/silva_db.tar.gz -O {database_path}silva_db.tar.gz;' \
                  f'tar zxvf {database_path}silva_db.tar.gz -C {database_path} > stdout;' \
                  f'gunzip {database_path}silva_db/*.fasta.gz;' \
                  f'echo "\t* Create index for bwa tools";' \
                  f'for fasta in {database_path}silva_db/*.fasta; do {path}/snakevir_env/bin/bwa index $fasta 1> stdout 2> stdout;done;' \
                  f'rm stdout {database_path}silva_db.tar.gz'
        list_skip.append(download(f'{database_path}silva_db/silva_138.1_bacteria.fasta.amb', command, database, skip))

        # Check if some files aren't download because they already exist
        if len([file_skiped for file_skiped in list_skip if file_skiped != None]) != 0:
            click.secho(f'\nSome file already exist, skip this file for download : \n', fg='green', bold=True)
            for file_skiped in list_skip:
                # Skipe None value from re-download file
                if file_skiped != None:
                    click.secho(f'\t* {file_skiped}\n', fg='green', bold=True)

def config_yml(path, database_path, install_path):
    """
    This function modify config.yaml with path of database install by install_database fuction
    """
    new_config = list()
    with open(f'{install_path}/config.yaml', 'r') as config:
        for line in config:
            if "rRNA_bact:" in line :
                line = f'rRNA_bact: "{database_path}silva_db/silva_138.1_bacteria.fasta"\n'
            if "rRNA_host:" in line:
                line = f'rRNA_host: "{database_path}silva_db/silva_138.1_insecta.fasta"\n'
            if "base_taxo:" in line:
                line = f'base_taxo: "{database_path}prot.accession2taxid"\n'
            if "base_taxo_nt:" in line:
                line = f'base_taxo_nt: "{database_path}nucl_gb.accession2taxid"\n'
            if "host_db:" in line:
                line = f'host_db: "{database_path}virushostdb.tsv"\n'
            if "Scripts:" in line:
                line = f'Scripts: "{Path(__file__).resolve().parent.parent}/script/"\n'
            if "module_file:" in line:
                line = f'module_file: "{path}/snakevir_module"\n'
            new_config.append(line)

    with open(f'{install_path}/config.yaml', 'w') as new_file:
        new_file.write("".join(new_config))

def module_file(path,install_path):
    """
    This function create module file for conda env
    """
    new_module = list()
    with open(f'{install_path}/snakevir_module','r') as module_file:
        for line in module_file:
            if "prepend-path PATH" in line:
                line = f"prepend-path PATH {path}/snakevir_env/bin\n"
            if "prepend-path LD_LIBRARY_PATH" in line:
                line = f"prepend-path LD_LIBRARY_PATH {path}/snakevir_env/lib\n"
            if "prepend-path CPATH" in line:
                line = f"prepend-path CPATH {path}/snakevir_env/include"
            new_module.append(line)

    with open(f'{path}/snakevir_module', 'w') as new_file:
        new_file.write("".join(new_module))

def __install(path, tool, database, skip):
    """
    This function allow to install tools with conda and download database needed by snakevir except nt & nr database
    """
    ### Install tools ###
    # Path to install file
    install_path = f'{Path(__file__).resolve().parent.parent}/install_files'
    # Function that install all tools with conda
    install_conda_env(path, install_path, tool, skip)

    ### Install database ###
    # Path to database file
    database_path = f'{path}/database/'
    install_database(path, database_path, install_path, database, skip)

    ### Create module file for cluster ###
    module_file(path,install_path)

    ### Modify config.yaml ###
    config_yml(path, database_path, install_path)

    ### Create profile for slurm cluster ###
    cookiecutter(template = f'https://github.com/Snakemake-Profiles/slurm.git',
                 no_input=True,
                 extra_context={ "sbatch_defaults": "--export=ALL",
                                "cluster_config": f"{install_path}/cluster.yaml",
                                },
                 overwrite_if_exists=True,
                 output_dir=f'{install_path}/profile/',
                 skip_if_file_exists=True)
    add_config_slurm = 'cluster-cancel: "scancel"\n' \
                       'restart-times: 0\n' \
                       'jobscript: "slurm-jobscript.sh"\n' \
                       'cluster: "slurm-submit.py"\n' \
                       'cluster-status: "slurm-status.py"\n' \
                       'max-jobs-per-second: 1\n' \
                       'max-status-checks-per-second: 1\n' \
                       'local-cores: 1\n' \
                       'jobs: 100\n' \
                       'use-envmodules: true\n' \
                       'latency-wait: 1296000\n' \
                       'printshellcmds: true'
    with open(f'{install_path}/profile/slurm/config.yaml', 'w') as f:
        f.write(add_config_slurm)

    ### Write file for verif installation in main command ###
    with open(f'{install_path}/.install',"w") as f:
        f.write('DONE')





import sys
import csv
import glob
import pandas as pd
from itertools import groupby



file=sys.argv[1]
fileout=sys.argv[2]

with open(file) as csvfile:
	count=pd.read_csv(csvfile)
	count['sum'] = count[list(count.iloc[:,2:].columns)].sum(axis=1)
	count=count[['contig', 'nuc_pos','sum']]
	count=count.sort_values(['contig', 'nuc_pos'], ascending=[True, True]) .reset_index(drop=True)
	col_sum_list = count['sum'].tolist()
liste_pos=[]

names=count['contig'].unique()
for names in names:
	indices = []
	df = count.loc[count.contig==names]
	col_sum_list = df['sum'].tolist()
	indexed_cross = enumerate(col_sum_list)
	key = lambda x: x[1] < 10
	for key, group in groupby(indexed_cross, key=key):
		if key:
			chunk = list(group)
			indices.append((chunk[0][0], chunk[-1][0]))  # extracting the indice
	for pos in indices:
		liste_pos.append([df.iat[pos[0],0],df.iat[pos[0],1],df.iat[pos[1],1]])

df = pd.DataFrame(liste_pos)
df.columns = ['contig', 'start','end']
df.to_csv(fileout, index=False,header=False)




#!/usr/bin/env python3

# Module import
import rich_click as click
from pathlib import Path
from .function.edit_cluster import __edit_cluster
from .function.install import __install
from .function.make_config import __make_config
from .function.run import __run

version = Path(__file__).parent.resolve().joinpath("VERSION").open("r").readline().strip()

click.rich_click.COMMAND_GROUPS = {
    "snakevir": [
        {
            "name": "Install",
            "commands": ["install_cluster", "make_config", "edit_cluster"],
        },
        {
            "name": "Run snakevir workflow",
            "commands": ["run"],
        },
    ]
}

@click.group(name=f"snakevir", invoke_without_command=True, no_args_is_help=True)
@click.version_option(version)
def main():
    """
    Documentation :  https://snakevir.readthedocs.io/en/latest/
    """

@click.command("install_cluster", short_help=f'Install snakevir on HPC cluster',
               context_settings=dict(max_content_width=800))
@click.option('--path', '-p',type=click.Path(exists=True, resolve_path=True), required=True,
              help="Give the installation PATH for conda environment that contains all the necessary tools for snakevir.")
@click.option('--skip', '-s', is_flag=True,
              help="Skip all install and download if it's already existing")
@click.option('--tool', '-t', is_flag=True,
              help=" Update conda environment (Re-install conda environment even if it's already install)")
@click.option('--database', '-d', is_flag=True,
              help="Update database (Re-download files even if it's already download)")
def install_cluster(path, tool, database, skip):
    """
    This function allow to install tools with conda and dowload database needed by snakevir except nt & nr database
    """
    __install(path, tool, database, skip)


click.rich_click.GROUP_ARGUMENTS_OPTIONS = True
click.rich_click.OPTION_GROUPS = {
    "snakevir make_config": [
        {
            "name": "Mandatory options",
            "options": ["--output"],
        },
        {
            "name": "PATH options",
            "options": ["--fastq", "--path_diamond_nr", "--path_blast_nt", "--host_genome", "--output_directory"],
        },
        {
            "name": "Advanced options",
            "options": ["--name", "--R1", "--R2", "--ext"],
        },
        {
            "name": "Adpater options",
            "options": ["--A3", "--A5"],
        },
    ]
}
@click.command("make_config", short_help=f'Create config file at yaml format',
               context_settings=dict(max_content_width=800))
@click.option('--output', '-o', type=click.Path(resolve_path=True), required=True,
              help="Path of the output file with '.yaml' extension (config.yml needed for snakevir.")
@click.option('--output_directory', '-d', type=click.Path(resolve_path=True),
              help="Path of the output directory for Snakevir's result.")
@click.option('--name', '-n', default="RUN_NAME",
              help="Name of run (ex : HNXXXXXX)")
@click.option('--fastq', '-f',  default="/PATH/TO/FASTQ/DIRECTORY/", type=click.Path(resolve_path=True),
              help="Path to the fastq directory")
@click.option('--host_genome', '-g',  default="/PATH/TO/FASTA/GENOME/", type=click.Path(resolve_path=True),
              help="Path to the genome host at fasta format")
@click.option('--r1', default="_1", show_default=True,
              help="Type of your R1 fastq files contains in FASTQ directory (for exemple : '_R1' or '_1', etc. )")
@click.option('--r2', default="_2", show_default=True,
              help="Type of your R2 fastq files contains in FASTQ directory (for exemple : '_R2' or '_2', etc. )")
@click.option('--ext', default=".fastq.gz", show_default=True,
              help=" Etension of your reads files in the FASTQ directory (for exemple : '.fastq.gz' or '.fq', etc.)")
@click.option('--path_diamond_nr', default="/PATH/TO/DIAMOND/NR/DATABASE", type=click.Path(resolve_path=True),
              help="Path to the diamond nr database")
@click.option('--path_blast_nt', default="/PATH/TO/BLAST/NT/DATABASE", type=click.Path(resolve_path=True),
              help="Path to the blast nt database")
@click.option('--A3', default="CAGCGGACGCCTATGTGATG", show_default=True,
              help="Sequence of Adapter in 3'")
@click.option('--A5', default="CATCACATAGGCGTCCGCTG", show_default=True,
              help="Sequence of Adapter in 5'")
def make_config(name, fastq, r1, r2, ext, path_diamond_nr, path_blast_nt, a3, a5, output, host_genome, output_directory):
    """
    The command make_config is used for create config fime at yaml format for snakevir. You have 2 choice, you can use arguement
    for write all information needed in config or you can only use some argument (-o is mandatory) and wirte in the file after
    the missing information.
    """
    __make_config(name, fastq, r1, r2, ext, path_diamond_nr, path_blast_nt, a3, a5, output, host_genome, output_directory)

@click.command("edit_cluster", short_help=f'Create cluster config file',
               context_settings=dict(max_content_width=800))
@click.option('--partition', '-p', default="False", type=str,
              help="Name of the default partition.")
@click.option('--account', '-a',  default="False", type=str,
              help="Name of you're account for launch job in cluster")
@click.option('--edit', '-e', is_flag=True,
              help="Edit cluster config for less/more ressources")
def edit_cluster(partition, account, edit):
    """
    The command make_config is used for create config fime at yaml format for snakevir. You have 2 choice, you can use arguement
    for write all information needed in config or you can only use some argument (-o is mandatory) and wirte in the file after
    the missing information.
    """
    __edit_cluster(partition, account, edit)

@click.command("run", short_help=f'Create cluster config file',
               context_settings={"ignore_unknown_options": True,"max_content_width" : 800})
@click.option('--config', '-c',  type=str, required=True,
              help="Path of config file")
@click.argument('other_snakemake_option', nargs=-1, type=click.UNPROCESSED)
def run(config, other_snakemake_option):
    """
    Run the snbakevir workflow.
    """
    __run(config, other_snakemake_option)


if Path(f'{Path(__file__).resolve().parent}/install_files/.install').exists():
    main.add_command(run)

main.add_command(install_cluster)
main.add_command(make_config)
main.add_command(edit_cluster)

if __name__ == '__main__':
    main()



import sys, os
from collections import Counter
import pandas as pd
import csv
import subprocess
import pickle

with open(sys.argv[1], 'rb') as handle:
	taxo_dict = pickle.load(handle)
with open(sys.argv[2], 'rb') as handle:
	taxo_custom = pickle.load(handle)

print("1")
Taxonomy_file=sys.argv[3]
output_stat=sys.argv[4]

Taxonomy=open(Taxonomy_file, 'r')
print(Taxonomy)


liste_species=[]
for lineage in Taxonomy:
	rank=lineage.replace('\n','').replace('\r','').split(",")
	
	if rank[0]=="tax_id":
		ranks=rank[0:1]+["Nucleic_acid"]+["genome"]+["cluster"]+rank[1:2]+rank[3:]+["how"]
		print(ranks)
	elif rank[2]=="Viruses":
		ranks=rank[0:1]+["NA"]+["NA"]+["NA"]+rank[1:2]+rank[3:]+["NA"]
	else:
		continue
	liste_species.append(ranks)
print("1")
for ranks in liste_species:
	index=liste_species.index(ranks)
	print(index)

def complet_ictv(liste_species):
	for ranks in liste_species:
		index=liste_species.index(ranks)

		if liste_species[index][11] in taxo_dict['species'] and liste_species[index][10]=='NA':
			liste_species[index][10]=taxo_dict['species'][liste_species[index][11]]

		if liste_species[index][10] in taxo_dict['genus'] and liste_species[index][9]=='NA':
			liste_species[index][9]=taxo_dict['genus'][liste_species[index][10]]
			
		if liste_species[index][9]in taxo_dict['family'] and liste_species[index][8]=='NA':
			liste_species[index][8]=taxo_dict['family'][liste_species[index][9]]

			
		if liste_species[index][8] in taxo_dict['order'] and liste_species[index][7]=='NA':
			liste_species[index][7]=taxo_dict['order'][liste_species[index][8]]

			
		if liste_species[index][7] in taxo_dict['clas'] and liste_species[index][6]=='NA':
			liste_species[index][6]=taxo_dict['clas'][liste_species[index][7]]

			
		if liste_species[index][6] in taxo_dict['phylum'] and liste_species[index][5]=='NA':
			liste_species[index][5]=taxo_dict['phylum'][liste_species[index][6]]

			
		if liste_species[index][5] in taxo_dict['kingdom'] and liste_species[index][4]=='NA':
			liste_species[index][4]=taxo_dict['kingdom'][liste_species[index][5]]

		if liste_species[index][9] in taxo_dict['fam2gen'] and liste_species[index][4]=='NA':
			liste_species[index][4]=taxo_dict['fam2gen'][liste_species[index][9]]
	
complet_ictv(liste_species)
print("ok ictv")
n=0
for ranks in liste_species:
	n+=1
	print(n,"/",len(liste_species))
	index=liste_species.index(ranks)
	if liste_species[index][10] not in taxo_dict['species'] and liste_species[index][11] in taxo_dict['taxshi']:
		if liste_species[index][10]=="NA" and  taxo_dict['taxshi'][liste_species[index][11]][6]!="NA":
			liste_species[index][10]=taxo_dict['taxshi'][liste_species[index][11]][6]
			liste_species[index][13]+=" | shi_genus "
		if liste_species[index][9]=="NA" and  taxo_dict['taxshi'][liste_species[index][11]][5]!="NA":
			liste_species[index][9]=taxo_dict['taxshi'][liste_species[index][11]][5]
			liste_species[index][13]+=" | shi_fam "
		if liste_species[index][8]=="NA" and  taxo_dict['taxshi'][liste_species[index][11]][4]!="NA":
			liste_species[index][8]=taxo_dict['taxshi'][liste_species[index][11]][4]
			liste_species[index][13]+=" | shi_order "
		if liste_species[index][7]=="NA" and  taxo_dict['taxshi'][liste_species[index][11]][3]!="NA":
			liste_species[index][7]=taxo_dict['taxshi'][liste_species[index][11]][3]
			liste_species[index][13]+=" | shi_class "
		if liste_species[index][6]=="NA" and  taxo_dict['taxshi'][liste_species[index][11]][2]!="NA":
			liste_species[index][6]=taxo_dict['taxshi'][liste_species[index][11]][2]
			liste_species[index][13]+=" | shi-phyl "
		if liste_species[index][5]=="NA" and  taxo_dict['taxshi'][liste_species[index][11]][1]!="NA":
			liste_species[index][5]=taxo_dict['taxshi'][liste_species[index][11]][1]
			liste_species[index][13]+=" | shi_king "
		if liste_species[index][4]=="NA" and  taxo_dict['taxshi'][liste_species[index][11]][0]!="NA":
			liste_species[index][4]=taxo_dict['taxshi'][liste_species[index][11]][0]
			liste_species[index][13]+=" | shi_clade "
	complet_ictv(liste_species)
	
	if liste_species[index][11] not in taxo_dict['species'] and liste_species[index][11] in taxo_custom['taxcustom']:
		
		if liste_species[index][10]=="NA" and  taxo_custom['taxcustom'][liste_species[index][11]][10]!="NA":
			liste_species[index][10]=taxo_custom['taxcustom'][liste_species[index][11]][10]
			liste_species[index][13]+=" | custom_genus "
			
		if liste_species[index][9]=="NA" and  taxo_custom['taxcustom'][liste_species[index][11]][8]!="NA":
			liste_species[index][9]=taxo_custom['taxcustom'][liste_species[index][11]][8]
			liste_species[index][13]+=" | custom_fam "
			
		if liste_species[index][8]=="NA" and  taxo_custom['taxcustom'][liste_species[index][11]][7]!="NA":
			liste_species[index][8]=taxo_custom['taxcustom'][liste_species[index][11]][7]
			liste_species[index][13]+=" | custom_order "
			
		if liste_species[index][7]=="NA" and  taxo_custom['taxcustom'][liste_species[index][11]][6]!="NA":
			liste_species[index][7]=taxo_custom['taxcustom'][liste_species[index][11]][6]
			liste_species[index][13]+=" | custom_class "
			
		if liste_species[index][6]=="NA" and  taxo_custom['taxcustom'][liste_species[index][11]][5]!="NA":
			liste_species[index][6]=taxo_custom['taxcustom'][liste_species[index][11]][5]
			liste_species[index][13]+=" | custom_phyl "
			
		if liste_species[index][5]=="NA" and  taxo_custom['taxcustom'][liste_species[index][11]][4]!="NA":
			liste_species[index][5]=taxo_custom['taxcustom'][liste_species[index][11]][4]
			liste_species[index][13]+=" | custom_king "
			
		if liste_species[index][4]=="NA" and  taxo_custom['taxcustom'][liste_species[index][11]][3]!="NA":
			liste_species[index][4]=taxo_custom['taxcustom'][liste_species[index][11]][3]
			liste_species[index][13]+=" | custom_clade "
		if liste_species[index][3]=="NA":
			liste_species[index][3]=taxo_custom['taxcustom'][liste_species[index][11]][2]

			liste_species[index][13]+=" | custom_clust "
		
	complet_ictv(liste_species)
		
	if liste_species[index][11] in taxo_dict['clustictv'] and liste_species[index][3]=='NA':
		liste_species[index][3]=taxo_dict['clustictv'][liste_species[index][11]]

	
	if liste_species[index][3]=='NA' and liste_species[index][9]!="NA":
		liste_species[index][3]=(liste_species[index][9].replace('viridae', ''))

	if liste_species[index][11] in taxo_dict['clustshi'] and liste_species[index][3]=='NA':
		liste_species[index][3]=taxo_dict['clustshi'][liste_species[index][11]]
		liste_species[index][13]+=" | clustshi "
		
	if liste_species[index][3]=='NA' and liste_species[index][8]!="NA":
		liste_species[index][3]=(liste_species[index][8].replace('virales', '')+"_unclass")
		
	if liste_species[index][1]=='NA' and liste_species[index][7] in taxo_dict['clas2gent']:
		liste_species[index][1]=taxo_dict['clas2gent'][liste_species[index][7]]
	if liste_species[index][1]=='NA' and liste_species[index][8] in taxo_dict['ord2gent']:
		liste_species[index][1]=taxo_dict['ord2gent'][liste_species[index][8]]
	if liste_species[index][1]=='NA' and liste_species[index][9] in taxo_dict['fam2gent']:
		liste_species[index][1]=taxo_dict['fam2gent'][liste_species[index][9]]

	if liste_species[index][1]=="NA":
		if "ssRNA negative-strand viruses" in liste_species[index][12] or "ssRNA negative-strand viruses" in   liste_species[index][4]:
			liste_species[index][1]="ssRNA(-)"
		if "ssRNA positive-strand viruses" in liste_species[index][12]or "ssRNA positive-strand viruses"  in   liste_species[index][4]:
			liste_species[index][1]="ssRNA(+)"
		if "dsRNA" in liste_species[index][12]or "dsRNA" in   liste_species[index][4]:
			liste_species[index][1]="dsRNA"
		if "RNA satellites" in liste_species[index][12] or "RNA satellites" in liste_species[index][4]:
			liste_species[index][1]="RNA satellites"
		if "ssDNA" in liste_species[index][12] or  "ssDNA"  in liste_species[index][4]:
			liste_species[index][1]="ssDNA(+/-)"
		if "dsDNA" in liste_species[index][12]  or  "dsDNA" in liste_species[index][4]:
			liste_species[index][1]="dsDNA"
		if "unclassified ssRNA" in liste_species[index][12] or "unclassified ssRNA"in liste_species[index][4]:
			liste_species[index][1]="ssRNA"
		if "unclassified RNA" in liste_species[index][12] or "unclassified RNA" in liste_species[index][4]:
	 		 liste_species[index][1]="RNA"
	if liste_species[index][2]=="NA":
		if 'DNA' in liste_species[index][1] :
			liste_species[index][2]="DNA"
		if 'RNA' in liste_species[index][1] :
			liste_species[index][2]="RNA"
	if liste_species[index][2]=="NA":
		if 'DNA' in liste_species[index][11] :
			liste_species[index][2]="DNA"
		if 'RNA' in liste_species[index][11] :
			liste_species[index][2]="RNA"
	
	if liste_species[index][2]=="NA":
		if "Riboviria" in liste_species[index][4]:
			liste_species[index][2]="RNA"
		if "Varidnaviria" in liste_species[index][4]:
			liste_species[index][2]="DNA"
		if "Duplodnaviria" in liste_species[index][4] or "Monodnaviria"in liste_species[index][4] :
			liste_species[index][2]="DNA"
			
	if liste_species[index][4]=="NA":
		if "RNA" in liste_species[index][2]:
			liste_species[index][4]="Riboviria"
		if "DNA" in liste_species[index][2]:
			liste_species[index][4]="Varidnaviria"
		if "DNA" in liste_species[index][4]:
			liste_species[index][4]="Duplodnaviria"
			
	if liste_species[index][6]=='NA' and liste_species[index][1]!='NA':
		liste_species[index][6]=liste_species[index][2]

print(liste_species)
with open(output_stat, "w") as f:
    writer = csv.writer(f)
    writer.writerows(liste_species)

